'From Squeak4.5 of 21 May 2014 [latest update: #13771] on 21 May 2014 at 9:19:01 pm'!
"Change Set:		LX-1
Date:			21 May 2014
Author:			David T. Lewis

Add LXDateAndTime and LXTimeStamp as alternative implementations of DateAndTime an TimeStamp.

DateAndTime uses instance variables 'seconds offset jdn nanos' and calculates its magnitude based on those varables in conjunction with the local TimeZone,

The LXDateAndTime variation uses instance variables 'utcMicroseconds localOffsetSeconds'. It represents time magnitude as elapsed microseconds since the Posix epoch, with localOffsetSeconds representing local offset from UTC. The magnitude is used for comparison and duration calculations, and the local offset is used for displaying this magnitude in the context of a local time zone.

The implementation ignores leap seconds, which are adjustments made to maintain earth rotational clock time in synchronization with elapsed seconds.

LXDateAndTime class>>now uses #primitiveUtcWithOffset to obtain current time in UTC microseconds with current local offset in seconds. The primitive provides an atomic query for UTC time and local offset as measured by the OS platform.  If primitiveUtcWithOffset is not available, the traditional implementation is used, which relies on a primitive for microseconds in the local time zone and derives UTC based on the TimeZone setting.

Tests are provided to verify equivalence of the LXDateAndTime and DateAndTime hierarchies. LXDateAndTimeConversionTest tests conversion to and from LXDateAndTime. LXDateAndTimeTest, LXDateAndTimeEpochTest, and LXDateAndTimeLeapTest are variations on their corresponding DateAndTime tests. These tests are placed in the Kernel package because they are temporary tests that will be removed when LXDateAndTime replaces DateAndTime in a later update.
"!

Magnitude subclass: #LXDateAndTime
	instanceVariableNames: 'utcMicroseconds localOffsetSeconds'
	classVariableNames: 'ClockProvider DaysSinceEpoch LastMilliSeconds LastTick LastTickSemaphore LocalTimeZone MilliSecondOffset OffsetsAreValid'
	poolDictionaries: 'ChronologyConstants'
	category: 'Kernel-Chronology'!

!LXDateAndTime commentStamp: 'dtl 3/31/2014 19:49' prior: 0!
I represent a point in UTC time as defined by ISO 8601. I have zero duration.

My magnitude represents the elapsed time since (or before) the Posix epoch.

I have a local offset for presenting my date and time in the context of a local time zone. This offset is independent of my magnitude.

My implementation ignores leap seconds, which are adjustments made to maintain earth rotational clock time in synchronization with elapsed seconds.!

TestCase subclass: #LXDateAndTimeConversionTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Kernel-Chronology'!
TestCase subclass: #LXDateAndTimeEpochTest
	instanceVariableNames: 'aDateAndTime aDuration aTimeZone localTimeZoneToRestore'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Kernel-Chronology'!

!LXDateAndTimeEpochTest commentStamp: 'tlk 1/6/2004 18:27' prior: 0!
I represent one of several Sunit test Cases intentended to provide complete coverage  for the Chronology set of classes as part of the external testing. The other Chronology sunit test cases are:
 DateTestCase
 DateAndTimeLeapTestCase,
 DurationTestCase,
 ScheduleTestCase
 TimeStampTestCase
 TimespanDoTestCase, 
 TimespanDoSpanAYearTestCase, 
 TimespanTestCase, 
 YearMonthWeekTestCase.  
These tests attempt to exercise all public and private methods.  Except, they do not explicitly depreciated methods. tlk
My fixtures are:
aDateAndTime = January 01, 1901 midnight (the start of the Squeak epoch) with localTimeZone = Grenwhich Meridian (local offset = 0 hours)
aDuration = 1 day, 2 hours, 3, minutes, 4 seconds and 5 nano seconds.
aTimeZone =  'Epoch Test Time Zone', 'ETZ' , offset: 12 hours, 15 minutes. !

TestCase subclass: #LXDateAndTimeLeapTest
	instanceVariableNames: 'aDateAndTime aDuration aTimeZone localTimeZoneToRestore'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Kernel-Chronology'!

!LXDateAndTimeLeapTest commentStamp: 'tlk 1/6/2004 17:54' prior: 0!
I represent one of several Sunit test Cases intentended to provide complete coverage for the Chronology set of classes as part of the external testing. tlk.
My fixtures are:
aDateAndTime = February 29, 2004 1:33 PM with offset: 2 hours
aDuration = 15 days, 14 hours, 13 minutes, 12 seconds and 11 nano seconds.
aTimeZone =  Grenwhich Meridian (local offset = 0 hours) !

ClassTestCase subclass: #LXDateAndTimeTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Kernel-Chronology'!
LXDateAndTime subclass: #LXTimeStamp
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Kernel-Chronology'!

!LXTimeStamp commentStamp: '<historical>' prior: 0!
This represents a duration of 0 length that marks a particular point in time.!


!DateAndTime methodsFor: 'squeak protocol' stamp: 'dtl 5/14/2014 07:37'!
floor
	"Answer a copy with magnitude rounded down to the nearest whole second"
	^self class basicNew
		ticks: (self ticks at: 3 put: 0; yourself)
		offset: offset.
! !

!DateAndTime methodsFor: 'LX-Kernel-Chronology' stamp: 'dtl 3/23/2014 13:40'!
asLXDateAndTime

	^LXDateAndTime
		year: self year
		month: self month
		day: self dayOfMonth
		hour: self hour
		minute: self minute
		second: self second
		nanoSecond: self nanoSecond
		offset: self offset! !


!LXDateAndTime methodsFor: 'ansi protocol' stamp: 'dtl 3/31/2014 21:10'!
+ operand
	"operand conforms to protocol Duration"

	^ self class
		utcMicroseconds: operand asDuration asNanoSeconds / 1000 + utcMicroseconds
		offset: localOffsetSeconds
! !

!LXDateAndTime methodsFor: 'ansi protocol' stamp: 'dtl 3/23/2014 13:58'!
- operand
	"operand conforms to protocol DateAndTime or protocol Duration"

	^ (operand respondsTo: #asLXDateAndTime)
		ifTrue: 
			[ | lticks rticks |
			lticks := self asLocal ticks.
	
		rticks := operand asLXDateAndTime asLocal ticks.
			Duration
 				seconds: (SecondsInDay *(lticks first - rticks first)) + 
							(lticks second - rticks second)
 				nanoSeconds: (lticks third - rticks third) ]
	
	ifFalse:
		
 	[ self + (operand negated) ]
! !

!LXDateAndTime methodsFor: 'ansi protocol' stamp: 'dtl 4/16/2014 18:58'!
< comparand
	"comparand conforms to protocol DateAndTime,
	or can be converted into something that conforms."

	^utcMicroseconds < comparand asLXDateAndTime utcMicroseconds
! !

!LXDateAndTime methodsFor: 'ansi protocol' stamp: 'dtl 3/23/2014 22:51'!
= aDateAndTimeOrTimeStamp
	"Equal if the absolute time values match, regardless of local time transform"
	self == aDateAndTimeOrTimeStamp ifTrue: [ ^ true ].
	((aDateAndTimeOrTimeStamp isKindOf: self class)
		or: [aDateAndTimeOrTimeStamp isKindOf: LXDateAndTime orOf: TimeStamp])
			ifFalse: [ ^ false ].
	^utcMicroseconds = aDateAndTimeOrTimeStamp utcMicroseconds! !

!LXDateAndTime methodsFor: 'ansi protocol' stamp: 'brp 8/23/2003 13:11'!
asLocal
	

	^ (self offset = self class localOffset)

		ifTrue: [self]
		ifFalse: [self utcOffset: self class localOffset]! !

!LXDateAndTime methodsFor: 'ansi protocol' stamp: 'dtl 3/23/2014 13:51'!
asUTC

	^ self offset isZero
		ifTrue: [self]
		ifFalse: [self utcOffset: 0]
! !

!LXDateAndTime methodsFor: 'ansi protocol' stamp: 'brp 8/23/2003 21:03'!
dayOfMonth
	"Answer which day of the month is represented by the receiver."

	^ self
		dayMonthYearDo: [ :d :m :y | d ]
! !

!LXDateAndTime methodsFor: 'ansi protocol' stamp: 'dtl 3/23/2014 17:32'!
dayOfWeek

	"Sunday=1, ... , Saturday=7"

	^ (self julianDayNumber + 1 rem: 7) + 1
! !

!LXDateAndTime methodsFor: 'ansi protocol' stamp: 'brp 8/23/2003 10:34'!
dayOfWeekAbbreviation

	^ self dayOfWeekName copyFrom: 1 to: 3
! !

!LXDateAndTime methodsFor: 'ansi protocol' stamp: 'brp 5/13/2003 07:28'!
dayOfWeekName

	^ Week nameOfDay: self dayOfWeek! !

!LXDateAndTime methodsFor: 'ansi protocol' stamp: 'nice 12/26/2009 01:29'!
dayOfYear
	"This code was contributed by Dan Ingalls. It is equivalent to the terser
		^ jdn - (Year year: self year) start julianDayNumber + 1 but much quicker."

	^ self dayMonthYearDo:
		[ :d :m :y |
			| monthStart |
			monthStart := #(1 32 60 91 121 152 182 213 244 274 305 335) at: m.
			(m > 2 and: [ Year isLeapYear: y ])
				ifTrue: [ monthStart + d ]
				ifFalse: [ monthStart + d - 1 ]]! !

!LXDateAndTime methodsFor: 'ansi protocol' stamp: 'dtl 3/23/2014 22:39'!
hash
	| totalSeconds |
	totalSeconds := self getSeconds - self offsetSeconds.
	^ ((totalSeconds // 86400 + self julianDayNumber) hashMultiply bitXor: totalSeconds \\
86400) bitXor: self nanoSecond! !

!LXDateAndTime methodsFor: 'ansi protocol' stamp: 'brp 5/13/2003 07:29'!
hour

	^ self hour24! !

!LXDateAndTime methodsFor: 'ansi protocol' stamp: 'avi 2/21/2004 18:46'!
hour12
	"Answer an <integer> between 1 and 12, inclusive, representing the hour 
	of the day in the 12-hour clock of the local time of the receiver."
	^ self hour24 - 1 \\ 12 + 1
! !

!LXDateAndTime methodsFor: 'ansi protocol' stamp: 'dtl 3/23/2014 17:59'!
hour24


	^ (Duration seconds: self getSeconds) hours! !

!LXDateAndTime methodsFor: 'ansi protocol' stamp: 'brp 5/13/2003 07:29'!
isLeapYear


	^ Year isLeapYear: self year
! !

!LXDateAndTime methodsFor: 'ansi protocol' stamp: 'brp 8/24/2003 11:03'!
meridianAbbreviation

	^ self asTime meridianAbbreviation
! !

!LXDateAndTime methodsFor: 'ansi protocol' stamp: 'dtl 3/23/2014 18:00'!
minute


	^ (Duration seconds: self getSeconds) minutes! !

!LXDateAndTime methodsFor: 'ansi protocol' stamp: 'brp 8/23/2003 21:05'!
month

	^ self 
		dayMonthYearDo: [ :d :m :y | m ]
! !

!LXDateAndTime methodsFor: 'ansi protocol' stamp: 'brp 5/13/2003 07:30'!
monthAbbreviation


	^ self monthName copyFrom: 1 to: 3! !

!LXDateAndTime methodsFor: 'ansi protocol' stamp: 'brp 5/13/2003 07:30'!
monthName


	^ Month nameOfMonth: self month! !

!LXDateAndTime methodsFor: 'ansi protocol' stamp: 'dtl 3/23/2014 22:59'!
offset

	^ Duration seconds: localOffsetSeconds! !

!LXDateAndTime methodsFor: 'ansi protocol' stamp: 'dtl 3/23/2014 22:56'!
offset: anOffset

	"Answer a <DateAndTime> equivalent to the receiver but with its local time 
	being offset from UTC by offset."

	^ self class utcMicroseconds: utcMicroseconds offset: anOffset asDuration asSeconds
! !

!LXDateAndTime methodsFor: 'ansi protocol' stamp: 'dtl 3/23/2014 18:00'!
second


	^ (Duration seconds: self getSeconds) seconds! !

!LXDateAndTime methodsFor: 'ansi protocol' stamp: 'brp 9/4/2003 06:42'!
timeZoneAbbreviation

	^ self class localTimeZone abbreviation! !

!LXDateAndTime methodsFor: 'ansi protocol' stamp: 'brp 9/4/2003 06:42'!
timeZoneName

	^ self class localTimeZone name! !

!LXDateAndTime methodsFor: 'ansi protocol' stamp: 'brp 8/23/2003 21:05'!
year
	^ self
		dayMonthYearDo: [ :d :m :y | y ]
! !

!LXDateAndTime methodsFor: 'squeak protocol' stamp: 'klc 9/12/2010 15:56'!
asDate


	^ Date starting: self asDateAndTime! !

!LXDateAndTime methodsFor: 'squeak protocol' stamp: 'dtl 3/23/2014 13:11'!
asDateAndTime

	^DateAndTime
		year: self year
		month: self month
		day: self dayOfMonth
		hour: self hour
		minute: self minute
		second: self second
		nanoSecond: self nanoSecond
		offset: self offset! !

!LXDateAndTime methodsFor: 'squeak protocol' stamp: 'dtl 3/23/2014 22:38'!
asDuration
	"Answer the duration since midnight."

	^ Duration seconds: self getSeconds nanoSeconds: self nanoSecond
! !

!LXDateAndTime methodsFor: 'squeak protocol' stamp: 'dtl 3/23/2014 12:31'!
asLXDateAndTime

	^ self! !

!LXDateAndTime methodsFor: 'squeak protocol' stamp: 'brp 5/13/2003 07:47'!
asMonth

	^ Month starting: self! !

!LXDateAndTime methodsFor: 'squeak protocol' stamp: 'brp 7/27/2003 15:45'!
asNanoSeconds
	"Answer the number of nanoseconds since midnight"

	^ self asDuration asNanoSeconds! !

!LXDateAndTime methodsFor: 'squeak protocol' stamp: 'dtl 3/23/2014 22:38'!
asTime


	^ Time seconds: self getSeconds nanoSeconds: self nanoSecond
! !

!LXDateAndTime methodsFor: 'squeak protocol' stamp: 'dtl 3/23/2014 13:20'!
asTimeStamp

	^ self
		asDateAndTime "FIXME LX hack for test support"
		as: TimeStamp! !

!LXDateAndTime methodsFor: 'squeak protocol' stamp: 'bf 8/28/2008 13:45'!
asUnixTime
	"answer number of seconds since unix epoch (midnight Jan 1, 1970, UTC)"

	^(self - self class unixEpoch) asSeconds! !

!LXDateAndTime methodsFor: 'squeak protocol' stamp: 'brp 5/13/2003 07:47'!
asWeek

	^ Week starting: self! !

!LXDateAndTime methodsFor: 'squeak protocol' stamp: 'brp 5/13/2003 07:47'!
asYear

	^ Year starting: self! !

!LXDateAndTime methodsFor: 'squeak protocol' stamp: 'dtl 3/23/2014 17:32'!
dayMonthYearDo: aBlock
	"Evaluation the block with three arguments: day month, year."

	| l n i j dd mm yyyy |
	l := self julianDayNumber + 68569.
	n := 4 * l // 146097.
	l := l - (146097 * n + 3 // 4).
	i := 4000 * (l + 1) // 1461001.
	l := l - (1461 * i // 4) + 31.
	j := 80 * l // 2447.
	dd := l - (2447 * j // 80).
	l := j // 11.
	mm := j + 2 - (12 * l).
	yyyy := 100 * (n - 49) + i + l.

	^ aBlock
		value: dd
		value: mm
		value: yyyy! !

!LXDateAndTime methodsFor: 'squeak protocol' stamp: 'brp 5/13/2003 07:49'!
duration

	^ Duration zero! !

!LXDateAndTime methodsFor: 'squeak protocol' stamp: 'dtl 4/8/2014 20:10'!
julianDayNumber

	| posixDays posixSeconds localSeconds negativeDays |
	posixSeconds := utcMicroseconds / 1000000.
	localSeconds := posixSeconds + localOffsetSeconds.
	negativeDays := 0.
	localSeconds < 0 ifTrue: [ "normalize"
			negativeDays := localSeconds // SecondsInDay.
			localSeconds := negativeDays - 1 * SecondsInDay + localSeconds].
	posixDays := (localSeconds / SecondsInDay) asInteger.
	^posixDays + self posixEpochJulianDays - negativeDays.
! !

!LXDateAndTime methodsFor: 'squeak protocol' stamp: 'dtl 3/23/2014 22:59'!
makeUTC
	"Make the receiver's timezone UTC."
	localOffsetSeconds := 0! !

!LXDateAndTime methodsFor: 'squeak protocol' stamp: 'brp 5/13/2003 07:49'!
middleOf: aDuration
	"Return a Timespan where the receiver is the middle of the Duration"

	| duration |
	duration := aDuration asDuration.

	^ Timespan starting: (self - (duration / 2)) duration: duration
! !

!LXDateAndTime methodsFor: 'squeak protocol' stamp: 'dtl 3/23/2014 17:33'!
midnight
	"Answer a DateAndTime starting at midnight of the same timezone offset as the receiver."
	^ self class basicNew
		setJdn: self julianDayNumber
		seconds: 0
		nano: 0
		offset: self offset! !

!LXDateAndTime methodsFor: 'squeak protocol' stamp: 'dtl 4/4/2014 20:19'!
nanoSecond

	^utcMicroseconds \\ 1000000 * 1000
! !

!LXDateAndTime methodsFor: 'squeak protocol' stamp: 'brp 7/27/2003 15:49'!
noon
	"Answer a DateAndTime starting at noon"

	^ self dayMonthYearDo: 
		[ :d :m :y | self class year: y month: m day: d hour: 12 minute: 0 second: 0 ]
! !

!LXDateAndTime methodsFor: 'squeak protocol' stamp: 'nk 3/12/2004 10:03'!
printHMSOn: aStream
	"Print just hh:mm:ss"
	aStream
		nextPutAll: (self hour asString padded: #left to: 2 with: $0);
		nextPut: $:;
		nextPutAll: (self minute asString padded: #left to: 2 with: $0);
		nextPut: $:;
		nextPutAll: (self second asString padded: #left to: 2 with: $0).
! !

!LXDateAndTime methodsFor: 'squeak protocol' stamp: 'nk 3/12/2004 10:38'!
printOn: aStream
	"Print as per ISO 8601 sections 5.3.3 and 5.4.1.
	Prints either:
		'YYYY-MM-DDThh:mm:ss.s+ZZ:zz:z' (for positive years) or '-YYYY-MM-DDThh:mm:ss.s+ZZ:zz:z' (for negative years)"

	^self printOn: aStream withLeadingSpace: false
! !

!LXDateAndTime methodsFor: 'squeak protocol' stamp: 'dtl 3/23/2014 14:42'!
printOn: aStream withLeadingSpace: printLeadingSpaceToo
	"Print as per ISO 8601 sections 5.3.3 and 5.4.1.
	If printLeadingSpaceToo is false, prints either:
		'YYYY-MM-DDThh:mm:ss.s+ZZ:zz:z' (for positive years) or '-YYYY-MM-DDThh:mm:ss.s+ZZ:zz:z' (for negative years)
	If printLeadingSpaceToo is true, prints either:
		' YYYY-MM-DDThh:mm:ss.s+ZZ:zz:z' (for positive years) or '-YYYY-MM-DDThh:mm:ss.s+ZZ:zz:z' (for negative years)
	"

	self printYMDOn: aStream withLeadingSpace: printLeadingSpaceToo.
	aStream nextPut: $T.
	self printHMSOn: aStream.
	self nanoSecond ~= 0 ifTrue:
		[ | z ps |
		ps := self nanoSecond printString padded: #left to: 9 with: $0.
		z := ps findLast: [ :c | c asciiValue > $0 asciiValue ].
		(z > 0) ifTrue: [aStream nextPut: $.].
		ps from: 1 to: z do: [ :c | aStream nextPut: c ] ].
	aStream
		nextPut: (self offset positive ifTrue: [$+] ifFalse: [$-]);
		nextPutAll: (self offset hours abs asString padded: #left to: 2 with: $0);
		nextPut: $:;
		nextPutAll: (self offset minutes abs asString padded: #left to: 2 with: $0).
	self offset seconds = 0 ifFalse:
		[ aStream
			nextPut: $:;
			nextPutAll: (self offset seconds abs truncated asString) ].
! !

!LXDateAndTime methodsFor: 'squeak protocol' stamp: 'nk 3/12/2004 10:29'!
printYMDOn: aStream
	"Print just YYYY-MM-DD part.
	If the year is negative, prints out '-YYYY-MM-DD'."

	^self printYMDOn: aStream withLeadingSpace: false.
! !

!LXDateAndTime methodsFor: 'squeak protocol' stamp: 'nk 3/12/2004 10:29'!
printYMDOn: aStream withLeadingSpace: printLeadingSpaceToo
	"Print just the year, month, and day on aStream.

	If printLeadingSpaceToo is true, then print as:
		' YYYY-MM-DD' (if the year is positive) or '-YYYY-MM-DD' (if the year is negative)
	otherwise print as:
		'YYYY-MM-DD' or '-YYYY-MM-DD' "

	| year month day |
	self dayMonthYearDo: [ :d :m :y | year := y. month := m. day := d ].
	year negative
		ifTrue: [ aStream nextPut: $- ]
		ifFalse: [ printLeadingSpaceToo ifTrue: [ aStream space ]].
	aStream
		nextPutAll: (year abs asString padded: #left to: 4 with: $0);
		nextPut: $-;
		nextPutAll: (month asString padded: #left to: 2 with: $0);
		nextPut: $-;
		nextPutAll: (day asString padded: #left to: 2 with: $0)
! !

!LXDateAndTime methodsFor: 'squeak protocol' stamp: 'dtl 3/23/2014 23:00'!
setJdn: jdn seconds: s nano: n offset: o

	localOffsetSeconds :=  o asSeconds.
	utcMicroseconds := self posixMicrosecondsJdn: jdn seconds: s nanos: n offset: localOffsetSeconds.

! !

!LXDateAndTime methodsFor: 'squeak protocol' stamp: 'dtl 3/23/2014 13:57'!
to: anEnd
	"Answer a Timespan. anEnd conforms to protocol DateAndTime or protocol Timespan"

	^ Timespan starting: self ending: (anEnd asLXDateAndTime)
! !

!LXDateAndTime methodsFor: 'squeak protocol' stamp: 'dtl 3/23/2014 13:57'!
to: anEnd by: aDuration
	"Answer a Timespan. anEnd conforms to protocol DateAndTime or protocol Timespan"

	^ (Schedule starting: self ending: (anEnd asLXDateAndTime))
		schedule: (Array with: aDuration asDuration);
		yourself
! !

!LXDateAndTime methodsFor: 'squeak protocol' stamp: 'brp 9/25/2003 16:01'!
to: anEnd by: aDuration do: aBlock
	"Answer a Timespan. anEnd conforms to protocol DateAndTime or protocol Timespan"

	^ (self to: anEnd by: aDuration) scheduleDo: aBlock! !

!LXDateAndTime methodsFor: 'squeak protocol' stamp: 'brp 8/23/2003 20:37'!
utcOffset: anOffset

	"Answer a <DateAndTime> equivalent to the receiver but offset from UTC by anOffset"

	| equiv |
	equiv := self + (anOffset asDuration - self offset).
	^ equiv ticks: (equiv ticks) offset: anOffset asDuration; yourself! !

!LXDateAndTime methodsFor: 'transitional - temporary' stamp: 'dtl 4/16/2014 19:20'!
asLXTimeStamp

	^ self as: LXTimeStamp! !

!LXDateAndTime methodsFor: 'transitional - temporary' stamp: 'dtl 3/23/2014 14:56'!
asPosixSeconds

	^ (self - (self class posixEpoch offset: self offset)) asSeconds! !

!LXDateAndTime methodsFor: 'transitional - temporary' stamp: 'dtl 4/9/2014 22:21'!
getSeconds

	| posixDays posixSeconds localSeconds |
	posixSeconds := utcMicroseconds / 1000000.
	localSeconds := posixSeconds + localOffsetSeconds.
	localSeconds < 0 ifTrue: [localSeconds := localSeconds \\ SecondsInDay]. "normalize"
	posixDays := (localSeconds / SecondsInDay) asInteger.
	^localSeconds - (posixDays * SecondsInDay).
! !

!LXDateAndTime methodsFor: 'transitional - temporary' stamp: 'dtl 3/23/2014 22:59'!
offsetSeconds

	^localOffsetSeconds ifNil: [localOffsetSeconds := self offset asSeconds]! !

!LXDateAndTime methodsFor: 'transitional - temporary' stamp: 'dtl 3/23/2014 22:44'!
posixMicrosecondsJdn: j seconds: s nanos: n offset: o

	| days totalSeconds micros |
	days := j - self posixEpochJulianDays.
	totalSeconds := days * 24 * 60 * 60 + s.
	micros := totalSeconds * 1000000.
	^micros + (n / 1000)
! !

!LXDateAndTime methodsFor: 'transitional - temporary' stamp: 'dtl 3/23/2014 22:51'!
utcMicroseconds
	^utcMicroseconds! !

!LXDateAndTime methodsFor: 'smalltalk-80' stamp: 'dtl 4/10/2014 08:07'!
asSeconds
	"Return the number of seconds since the Squeak epoch"

	self flag: #FIXME. "is this whole seconds or partial? UTC or local?"

	^ utcMicroseconds / 1000000 + self class epochOffset
! !

!LXDateAndTime methodsFor: 'smalltalk-80' stamp: 'brp 7/1/2003 17:53'!
day

	^ self dayOfYear
! !

!LXDateAndTime methodsFor: 'smalltalk-80' stamp: 'brp 5/13/2003 07:48'!
daysInMonth
	"Answer the number of days in the month represented by the receiver."


	^ self asMonth daysInMonth! !

!LXDateAndTime methodsFor: 'smalltalk-80' stamp: 'brp 5/13/2003 07:48'!
daysInYear

	"Answer the number of days in the year represented by the receiver."

	^ self asYear daysInYear! !

!LXDateAndTime methodsFor: 'smalltalk-80' stamp: 'brp 7/27/2003 15:44'!
daysLeftInYear
	"Answer the number of days in the year after the date of the receiver."

	^ self daysInYear - self dayOfYear! !

!LXDateAndTime methodsFor: 'smalltalk-80' stamp: 'brp 7/27/2003 15:44'!
firstDayOfMonth

	^ self asMonth start day
! !

!LXDateAndTime methodsFor: 'smalltalk-80' stamp: 'brp 7/1/2003 18:30'!
hours

	^ self hour
! !

!LXDateAndTime methodsFor: 'smalltalk-80' stamp: 'brp 1/7/2004 15:45'!
minutes

	^ self minute
! !

!LXDateAndTime methodsFor: 'smalltalk-80' stamp: 'brp 5/13/2003 07:50'!
monthIndex


	^ self month! !

!LXDateAndTime methodsFor: 'smalltalk-80' stamp: 'brp 7/1/2003 18:31'!
seconds

	^ self second
! !

!LXDateAndTime methodsFor: 'private' stamp: 'dtl 3/23/2014 22:38'!
hasEqualTicks: aDateAndTime
	
	^ (self julianDayNumber = aDateAndTime julianDayNumber)
		and: [ (self getSeconds = aDateAndTime secondsSinceMidnight)
			and: [ self nanoSecond = aDateAndTime nanoSecond ] ]

! !

!LXDateAndTime methodsFor: 'private' stamp: 'dtl 4/9/2014 22:24'!
normalize: i ticks: ticks base: base

	| tick div quo rem |
	tick := ticks at: i.
	div := tick asInteger digitDiv: base neg: tick negative.
	quo := (div at: 1) normalize.
	rem := (div at: 2) normalize.
	rem < 0 ifTrue: [ quo := quo - 1. rem := base + rem ].
	ticks at: (i-1) put: ((ticks at: i-1) + quo).
	ticks at: i put: rem
! !

!LXDateAndTime methodsFor: 'private' stamp: 'dtl 3/23/2014 18:00'!
secondsSinceMidnight

	^ self getSeconds! !

!LXDateAndTime methodsFor: 'private' stamp: 'dtl 3/23/2014 22:39'!
ticks
	"Private - answer an array with our instance variables. Assumed to be UTC "

	^ Array with: self julianDayNumber with: self getSeconds with: self nanoSecond
! !

!LXDateAndTime methodsFor: 'private' stamp: 'dtl 3/23/2014 23:00'!
ticks: ticks offset: utcOffset
	"ticks is {julianDayNumber. secondCount. nanoSeconds}"

	| jdn s nanos |
	self normalize: 3 ticks: ticks base: NanosInSecond.
	self normalize: 2 ticks: ticks base: SecondsInDay.

	jdn	:= ticks at: 1.
	s := ticks at: 2.
	nanos := ticks at: 3.
	localOffsetSeconds := utcOffset asSeconds.
	utcMicroseconds := self posixMicrosecondsJdn: jdn seconds: s nanos: nanos offset: localOffsetSeconds.
! !

!LXDateAndTime methodsFor: 'DTL-initializing' stamp: 'dtl 3/28/2014 07:53'!
posixEpochJulianDays

	^self class daysFromSmalltalkEpochToPosixEpoch + SqueakEpoch! !

!LXDateAndTime methodsFor: 'DTL-initializing' stamp: 'dtl 3/24/2014 16:35'!
utcMicroseconds: microsecondsSincePosixEpoch offset: tzOffset

	utcMicroseconds := microsecondsSincePosixEpoch.
	localOffsetSeconds := tzOffset.
! !


!LXDateAndTime class methodsFor: 'clock provider' stamp: 'kph 12/11/2006 20:14'!
clock 
	 "the provider of real time seconds/milliseconds."

	^ ClockProvider ! !

!LXDateAndTime class methodsFor: 'ansi protocol' stamp: 'gk 8/31/2006 00:49'!
clockPrecision
	"One nanosecond precision"

	^ Duration seconds: 0 nanoSeconds: 1
! !

!LXDateAndTime class methodsFor: 'ansi protocol' stamp: 'dtl 3/24/2014 16:30'!
now

	[ | timeArray |
	timeArray := self primPosixMicrosecondClockWithOffset.
	^ self utcMicroseconds: timeArray first offset: timeArray second]
		on: Error
		do: [ "Use old style primitive support"
			^self nowWithOffset: self localOffset]! !

!LXDateAndTime class methodsFor: 'ansi protocol' stamp: 'brp 7/27/2003 15:25'!
year: year day: dayOfYear hour: hour minute: minute second: second

	^ self
		year: year
		day: dayOfYear
		hour: hour
		minute: minute
		second: second
		offset: self localOffset
! !

!LXDateAndTime class methodsFor: 'ansi protocol' stamp: 'brp 7/27/2003 15:28'!
year: year day: dayOfYear hour: hour minute: minute second: second offset: offset 
	"Return a DataAndTime"

	| y d |
	y := self
		year: year
		month: 1
		day: 1
		hour: hour
		minute: minute
		second: second
		nanoSecond: 0
		offset: offset.

	d := Duration days: (dayOfYear - 1).

	^ y + d
! !

!LXDateAndTime class methodsFor: 'ansi protocol' stamp: 'brp 8/23/2003 21:00'!
year: year month: month day: day hour: hour minute: minute second: second
	"Return a DateAndTime"

	^ self
		year: year
		month: month
		day: day
		hour: hour
		minute: minute
		second: second
		offset: self localOffset! !

!LXDateAndTime class methodsFor: 'ansi protocol' stamp: 'brp 5/13/2003 07:36'!
year: year month: month day: day hour: hour minute: minute second: second offset: offset

	^ self
		year: year
		month: month
		day: day
		hour: hour
		minute: minute
		second: second
		nanoSecond: 0
		offset: offset! !

!LXDateAndTime class methodsFor: 'squeak protocol' stamp: 'brp 5/13/2003 07:36'!
current


	^ self now! !

!LXDateAndTime class methodsFor: 'squeak protocol' stamp: 'bf 7/3/2012 23:35'!
date: aDate time: aTime

	^ self 
		year: aDate year 
		day: aDate dayOfYear 
		hour: aTime hour 
		minute: aTime minute 
		second: aTime second
		offset: aDate start offset! !

!LXDateAndTime class methodsFor: 'squeak protocol' stamp: 'dtl 3/28/2014 18:44'!
epoch
	"Answer a DateAndTime representing the Squeak epoch: 1 January 1901"

	| uSec |
	self flag: #FIXME. "see comment in fromSeconds: "
	uSec := self epochOffsetMicros negated.
	^ self utcMicroseconds: uSec offset: self localOffsetSeconds


"
	^ self julianDayNumber: SqueakEpoch
"! !

!LXDateAndTime class methodsFor: 'squeak protocol' stamp: 'brp 5/13/2003 07:36'!
fromString: aString


	^ self readFrom: (ReadStream on: aString)! !

!LXDateAndTime class methodsFor: 'squeak protocol' stamp: 'cmm 2/2/2012 19:14'!
julianDayNumber: anInteger 
	^ self
		julianDayNumber: anInteger
		offset: self localOffset! !

!LXDateAndTime class methodsFor: 'squeak protocol' stamp: 'cmm 2/2/2012 19:13'!
julianDayNumber: anInteger offset: aDuration 
	^ self basicNew
		ticks: anInteger days ticks
		offset: aDuration ;
		 yourself! !

!LXDateAndTime class methodsFor: 'squeak protocol' stamp: 'brp 9/4/2003 06:40'!
localOffset
	"Answer the duration we are offset from UTC"

	^ self localTimeZone offset
! !

!LXDateAndTime class methodsFor: 'squeak protocol' stamp: 'brp 7/27/2003 17:09'!
midnight

	^ self now midnight! !

!LXDateAndTime class methodsFor: 'squeak protocol' stamp: 'ul 11/6/2010 23:38'!
milliSecondsSinceMidnight
	| msm msClock |

	"This is usually only during system startup..."
	self waitForOffsets.

	msClock := self millisecondClockValue.
	msClock < LastMilliSeconds ifTrue: [ "rolled over"
		MilliSecondOffset := MilliSecondOffset + (SmallInteger maxVal // 2) + 1 ].
	LastMilliSeconds := msClock.
	[
	msm := msClock + MilliSecondOffset.
	msm >= 86400000 ] whileTrue: [
		"next day"
		LastTick := -1.
		DaysSinceEpoch := DaysSinceEpoch + 1.
		MilliSecondOffset := MilliSecondOffset - 86400000 ].
	"day rolled over sanity check"
	(LastTick = -1 and: [
		(Duration
			days: SqueakEpoch
			hours: 0
			minutes: 0
			seconds: self clock totalSeconds) days ~= DaysSinceEpoch ]) ifTrue: [
		self initializeOffsets.
		^ self milliSecondsSinceMidnight ].
	^ msm.! !

!LXDateAndTime class methodsFor: 'squeak protocol' stamp: 'brp 8/23/2003 20:57'!
new
	"Answer a DateAndTime representing the Squeak epoch: 1 January 1901"

	^ self epoch! !

!LXDateAndTime class methodsFor: 'squeak protocol' stamp: 'brp 7/27/2003 17:09'!
noon

	^ self now noon
! !

!LXDateAndTime class methodsFor: 'squeak protocol' stamp: 'cmm 7/4/2012 13:42'!
nowWithOffset: aDuration 
	| nanoTicks msm |
	nanoTicks := (msm := self milliSecondsSinceMidnight) * 1000000.
	"The following usually only executes at system startup."
	^ LastTick < nanoTicks
		ifTrue:
			[ LastTick := nanoTicks.
			self waitForOffsets.
			self basicNew
				setJdn: DaysSinceEpoch
				seconds: msm // 1000
				nano: msm \\ 1000 * 1000000
				offset: aDuration ]
		ifFalse:
			[ LastTickSemaphore critical:
				[ LastTick := LastTick + 1.
				self waitForOffsets.
				self basicNew
					setJdn: DaysSinceEpoch
					seconds: LastTick // 1000000000
					nano: LastTick \\ 1000000000
					offset: aDuration ] ]
" 
[ 10000 timesRepeat: [ self now. ] ] timeToRun / 10000.0 . 

If calls to DateAndTime-c-#now are within a single millisecond the semaphore code 
to ensure that (self now <= self now) slows things down considerably by a factor of about 20.

The actual speed of a single call to DateAndTime-now in milliseconds is 
demonstrated by the unguarded method below.

[ 100000 timesRepeat: [ self todayAtMilliSeconds: (self milliSecondsSinceMidnight) ] ] timeToRun / 100000.0 .  0.00494 0.00481 0.00492 0.00495
  
"! !

!LXDateAndTime class methodsFor: 'squeak protocol' stamp: 'bf 3/4/2013 15:17'!
readFrom: aStream

	| offset date time ch |

	date := Date readFrom: aStream.
	[aStream peek isDigit]
		whileFalse: [aStream next].
	time := Time readFrom: aStream.
	(aStream atEnd or: [('+-Z' includes: aStream peek) not])
		ifTrue: [offset := self localOffset]
		ifFalse: [(aStream peekFor: $Z)
			ifTrue: [offset := Duration zero]
			ifFalse: [
				ch := aStream next.
				ch = $+ ifTrue: [ch := Character space].
				offset := Duration fromString: ch asString, '0:', aStream upToEnd, ':0']].
	^ self
		year: date year
		month: date monthIndex
		day: date dayOfMonth
		hour: time hour
		minute: time minute
		second: time second
		nanoSecond: time nanoSecond
		offset: offset


	"	'-1199-01-05T20:33:14.321-05:00' asDateAndTime
		' 2002-05-16T17:20:45.1+01:01' asDateAndTime

		' 2002-05-16T17:20:45.02+01:01' asDateAndTime

		' 2002-05-16T17:20:45.003+01:01' asDateAndTime

		' 2002-05-16T17:20:45.0004+01:01' asDateAndTime
  		' 2002-05-16T17:20:45.00005' asDateAndTime
		' 2002-05-16T17:20:45.000006+01:01' asDateAndTime

		' 2002-05-16T17:20:45.0000007+01:01' asDateAndTime
		' 2002-05-16T17:20:45.00000008-01:01' asDateAndTime   
		' 2002-05-16T17:20:45.000000009+01:01' asDateAndTime  
		' 2002-05-16T17:20:45.0000000001+01:01' asDateAndTime  

 		' 2002-05-16T17:20' asDateAndTime
		' 2002-05-16T17:20:45' asDateAndTime
		' 2002-05-16T17:20:45+01:57' asDateAndTime
 		' 2002-05-16T17:20:45-02:34' asDateAndTime
 		' 2002-05-16T17:20:45+00:00' asDateAndTime
		' 1997-04-26T01:02:03+01:02:3' asDateAndTime 
 	"! !

!LXDateAndTime class methodsFor: 'squeak protocol' stamp: 'brp 7/27/2003 17:09'!
today

	^ self midnight! !

!LXDateAndTime class methodsFor: 'squeak protocol' stamp: 'dtl 3/23/2014 13:42'!
tomorrow

	^ self today asDate next asLXDateAndTime
! !

!LXDateAndTime class methodsFor: 'squeak protocol' stamp: 'bf 8/28/2008 13:19'!
unixEpoch
	"Answer a DateAndTime representing the Unix epoch (1 January 1970, midnight UTC)"

	^ self basicNew
		ticks: #(2440588 0 0) offset: Duration zero;
		yourself.
! !

!LXDateAndTime class methodsFor: 'squeak protocol' stamp: 'brp 8/23/2003 20:53'!
year: year day: dayOfYear
	"Return a DateAndTime"

	^ self
		year: year
		day: dayOfYear
		hour: 0
		minute: 0
		second: 0
! !

!LXDateAndTime class methodsFor: 'squeak protocol' stamp: 'ul 10/15/2009 01:05'!
year: year month: month day: day
	"Return a DateAndTime, midnight local time"
	
	^self
 		year: year
 		month: month
 		day: day
 		hour: 0
		minute: 0
! !

!LXDateAndTime class methodsFor: 'squeak protocol' stamp: 'ul 10/15/2009 01:19'!
year: year month: month day: day hour: hour minute: minute
	"Return a DateAndTime"

	^self
 		year: year
 		month: month
 		day: day
 		hour: hour
		minute: minute
		second: 0
! !

!LXDateAndTime class methodsFor: 'squeak protocol' stamp: 'dtl 3/23/2014 20:50'!
year: year month: month day: day hour: hour minute: minute second: second nanoSecond: nanoCount offset: offset
	"Return a DateAndTime"

	| monthIndex daysInMonth p q r s julianDayNumber |

	monthIndex := month isInteger ifTrue: [month] ifFalse: [Month indexOfMonth: month].
	daysInMonth := Month
		daysInMonth: monthIndex
		forYear: year.
	day < 1 ifTrue: [self error: 'day may not be zero or negative'].
	day > daysInMonth ifTrue: [self error: 'day is after month ends']. 	
	
	p := (monthIndex - 14) quo: 12.
	q := year + 4800 + p.
	r := monthIndex - 2 - (12 * p).
	s := (year + 4900 + p) quo: 100.

	julianDayNumber :=
 		( (1461 * q) quo: 4 ) +
			( (367 * r) quo: 12 ) -
 				( (3 * s) quo: 4 ) +
 					( day - 32075 ).

	^self basicNew
		setJdn: julianDayNumber 
		seconds: hour * 60 + minute * 60 + second - offset asSeconds
		nano: nanoCount
		offset: offset;
		yourself! !

!LXDateAndTime class methodsFor: 'squeak protocol' stamp: 'dtl 3/23/2014 13:57'!
yesterday

	^ self today asDate previous asLXDateAndTime! !

!LXDateAndTime class methodsFor: 'DTL' stamp: 'dtl 3/28/2014 07:53'!
daysFromSmalltalkEpochToPosixEpoch

	^52 * 365 + (17 * 366)! !

!LXDateAndTime class methodsFor: 'DTL' stamp: 'dtl 4/10/2014 08:02'!
epochOffset
	"Elaspsed seconds from the Smalltalk epoch to the Posix epoch"
	^self daysFromSmalltalkEpochToPosixEpoch * SecondsInDay! !

!LXDateAndTime class methodsFor: 'DTL' stamp: 'dtl 3/28/2014 18:44'!
epochOffsetMicros
	"Elaspsed microseconds from the Smalltalk epoch to the Posix epoch"
	^self epochOffset * 1000000! !

!LXDateAndTime class methodsFor: 'DTL' stamp: 'dtl 3/28/2014 08:05'!
localOffsetSeconds
	^self localOffset asSeconds! !

!LXDateAndTime class methodsFor: 'DTL' stamp: 'dtl 3/23/2014 14:56'!
posixEpoch
	"Answer a DateAndTime representing the Posix epoch"

	^ self julianDayNumber: self posixEpochJulianDays ! !

!LXDateAndTime class methodsFor: 'DTL' stamp: 'dtl 3/24/2014 16:30'!
primPosixMicrosecondClockWithOffset
	"Answer an array with UTC microseconds since the Posix epoch and the
	current seconds offset from GMT in the local time zone."

	<primitive: 'primitiveUtcWithOffset'>

	^self primitiveFailed! !

!LXDateAndTime class methodsFor: 'DTL' stamp: 'dtl 3/23/2014 22:55'!
utcMicroseconds: microsecondsSincePosixEpoch offset: offsetSeconds

	^super new
		utcMicroseconds: microsecondsSincePosixEpoch
		offset: offsetSeconds! !

!LXDateAndTime class methodsFor: 'smalltalk-80' stamp: 'dtl 3/28/2014 18:49'!
fromSeconds: seconds 
	"Answer a DateAndTime since the Squeak epoch: 1 January 1901"

	"dtl - Implemented for compatibiity with Squeak, but this is probably wrong"

	| s uSec offset |
	self flag: #FIXME. "local versus UTC? what should we set the offset to?"
	offset := self localOffsetSeconds.
	s := seconds - self epochOffset - offset..
	uSec := s * 1000000.
	^ self utcMicroseconds: uSec offset: offset
! !

!LXDateAndTime class methodsFor: 'smalltalk-80' stamp: 'kph 12/11/2006 21:13'!
millisecondClockValue

	^ self clock millisecondClockValue! !

!LXDateAndTime class methodsFor: 'smalltalk-80' stamp: 'kph 12/9/2006 23:46'!
totalSeconds

	^ self clock totalSeconds! !

!LXDateAndTime class methodsFor: 'initialize-release' stamp: 'dtl 7/13/2009 13:08'!
initialize

	super initialize.

	ClockProvider := Time.
	LastTickSemaphore := Semaphore forMutualExclusion.
	LastMilliSeconds := 0.
	LastTick := 0.
	Smalltalk addToStartUpList: self.
	self startUp: true
! !

!LXDateAndTime class methodsFor: 'initialize-release' stamp: 'ul 11/6/2010 23:37'!
initializeOffsets
	| durationSinceEpoch secondsSinceMidnight nowSecs |
	LastTick := 0.
	nowSecs := self clock secondsWhenClockTicks.
	LastMilliSeconds := self millisecondClockValue.
	durationSinceEpoch := Duration
		days: SqueakEpoch
		hours: 0
		minutes: 0
		seconds: nowSecs.
	DaysSinceEpoch := durationSinceEpoch days.
	secondsSinceMidnight := (durationSinceEpoch -
		(Duration
			days: DaysSinceEpoch
			hours: 0
			minutes: 0
			seconds: 0)) asSeconds.
	MilliSecondOffset := secondsSinceMidnight * 1000 - LastMilliSeconds! !

!LXDateAndTime class methodsFor: 'initialize-release' stamp: 'dtl 4/13/2011 08:10'!
startUp: resuming
	resuming ifFalse: [ ^ self ].
	Time initializeMillisecondClockMask.
	OffsetsAreValid := false.
	[
		self initializeOffsets.
		OffsetsAreValid := true
	] forkAt: Processor userInterruptPriority.! !

!LXDateAndTime class methodsFor: 'initialize-release' stamp: 'jmv 10/19/2010 14:10'!
waitForOffsets
	OffsetsAreValid ifFalse: [
		[
			(Delay forSeconds: 1) wait.
			OffsetsAreValid
		] whileFalse
	]! !

!LXDateAndTime class methodsFor: 'accessing' stamp: 'brp 9/4/2003 06:39'!
localTimeZone
	"Answer the local time zone"

	^ LocalTimeZone ifNil: [ LocalTimeZone := TimeZone default ]

! !

!LXDateAndTime class methodsFor: 'accessing' stamp: 'nk 3/30/2004 09:53'!
localTimeZone: aTimeZone
	"Set the local time zone"

	"
	DateAndTime localTimeZone: (TimeZone offset:  0 hours name: 'Universal Time' abbreviation: 'UTC').
	DateAndTime localTimeZone: (TimeZone offset: -8 hours name: 'Pacific Standard Time' abbreviation: 'PST').
	"

	LocalTimeZone := aTimeZone


! !


!LXDateAndTimeConversionTest methodsFor: 'testing' stamp: 'dtl 4/5/2014 10:12'!
testAsSeconds

	| day dt hour lxdt min mo nano offset sec yr expectedSeconds |
	yr := 1970.
	mo := 12.
	day := 2.
	hour := 2.
	min := 3.
	sec := 4.
	nano := 5.
	offset := 6 hours.
	lxdt := LXDateAndTime 
			year: yr month: mo day: day 
			hour: hour minute: min second: sec nanoSecond: nano 
			offset: offset.
	dt := DateAndTime 
			year: yr month: mo day: day 
			hour: hour minute: min second: sec nanoSecond: nano 
			offset: offset.
	self assert: 2206404184 equals: dt asSeconds.
	self assert: 2206404184 equals: lxdt asSeconds.

	yr := 1969.
	lxdt := LXDateAndTime 
			year: yr month: mo day: day 
			hour: hour minute: min second: sec nanoSecond: nano 
			offset: offset.
	dt := DateAndTime 
			year: yr month: mo day: day 
			hour: hour minute: min second: sec nanoSecond: nano 
			offset: offset.
	expectedSeconds := 2206404184 - (365 * 24 * 60 * 60).
	self assert: expectedSeconds equals: dt asSeconds.
	self assert: expectedSeconds equals: lxdt asSeconds.
! !

!LXDateAndTimeConversionTest methodsFor: 'testing' stamp: 'dtl 4/9/2014 22:30'!
testAsSeconds2
	"(self selector: #testAsSeconds2) debug"

	| day dt hour lxdt min mo nano offset sec |
	mo := 12.
	day := 2.
	hour := 2.
	min := 3.
	sec := 4.
	nano := 5.
	offset := 6 hours.
"	-2000 to: 5000 do: [:yr |"
	1970 to: 5000 do: [:yr |
		lxdt := LXDateAndTime 
			year: yr month: mo day: day 
			hour: hour minute: min second: sec nanoSecond: nano 
			offset: offset.
		dt := DateAndTime 
			year: yr month: mo day: day 
			hour: hour minute: min second: sec nanoSecond: nano 
			offset: offset.
		self assert: dt asSeconds equals: lxdt asSeconds].
! !

!LXDateAndTimeConversionTest methodsFor: 'testing' stamp: 'dtl 4/8/2014 20:43'!
testDayOfMonth
	"(self selector: #testDayOfMonth) debug"

	| day dt hour lxdt min mo nano offset sec |
	mo := 12.
	day := 2.
	hour := 2.
	min := 3.
	sec := 4.
	nano := 5.
	offset := 6 hours.
	-1200 to: 3000 do: [:yr |
		lxdt := LXDateAndTime 
			year: yr month: mo day: day 
			hour: hour minute: min second: sec nanoSecond: nano 
			offset: offset.
		dt := DateAndTime 
			year: yr month: mo day: day 
			hour: hour minute: min second: sec nanoSecond: nano 
			offset: offset.
		self assert: dt dayOfMonth equals: lxdt dayOfMonth].
! !

!LXDateAndTimeConversionTest methodsFor: 'testing' stamp: 'dtl 4/8/2014 20:42'!
testDayOfWeek
	"(self selector: #testDayOfWeek) debug"

	| day dt hour lxdt min mo nano offset sec |
	mo := 12.
	day := 2.
	hour := 2.
	min := 3.
	sec := 4.
	nano := 5.
	offset := 6 hours.
	-1200 to: 3000 do: [:yr |
		lxdt := LXDateAndTime 
			year: yr month: mo day: day 
			hour: hour minute: min second: sec nanoSecond: nano 
			offset: offset.
		dt := DateAndTime 
			year: yr month: mo day: day 
			hour: hour minute: min second: sec nanoSecond: nano 
			offset: offset.
		self assert: dt dayOfWeek equals: lxdt dayOfWeek].
! !

!LXDateAndTimeConversionTest methodsFor: 'testing' stamp: 'dtl 4/8/2014 20:42'!
testDayOfYear
	"(self selector: #testDayOfYear) debug"

	| day dt hour lxdt min mo nano offset sec |
	mo := 12.
	day := 2.
	hour := 2.
	min := 3.
	sec := 4.
	nano := 5.
	offset := 6 hours.
	-1200 to: 3000 do: [:yr |
		lxdt := LXDateAndTime 
			year: yr month: mo day: day 
			hour: hour minute: min second: sec nanoSecond: nano 
			offset: offset.
		dt := DateAndTime 
			year: yr month: mo day: day 
			hour: hour minute: min second: sec nanoSecond: nano 
			offset: offset.
		self assert: dt dayOfYear equals: lxdt dayOfYear].
! !

!LXDateAndTimeConversionTest methodsFor: 'testing' stamp: 'dtl 4/12/2014 12:53'!
testFromString
	"Convert to string then back, verify same date and time. Skip years in the
	range 0 to 99 because they are interpreted relative to 2000."

	"(LXDateAndTimeConversionTest selector: #testFromString) debug"

	| day hour lxdt min mo nano offset sec newLxdt s |
	mo := 12.
	day := 2.
	hour := 2.
	min := 3.
	sec := 4.
	nano := 5.
	offset := 6 hours.
	(-2000 to: -1) , (100 to: 5000) do: [:yr |
		lxdt := LXDateAndTime 
			year: yr month: mo day: day 
			hour: hour minute: min second: sec nanoSecond: nano 
			offset: offset.
		s := lxdt asString.
		newLxdt := LXDateAndTime fromString: s.
		self assert: lxdt equals: newLxdt].
! !

!LXDateAndTimeConversionTest methodsFor: 'testing' stamp: 'dtl 4/8/2014 20:44'!
testJulianDayNumber
	"(self selector: #testJulianDayNumber) debug"

	| day dt hour lxdt min mo nano offset sec |
	mo := 12.
	day := 2.
	hour := 2.
	min := 3.
	sec := 4.
	nano := 5.
	offset := 6 hours.
	-2000 to: 5000 do: [:yr |
		lxdt := LXDateAndTime 
			year: yr month: mo day: day 
			hour: hour minute: min second: sec nanoSecond: nano 
			offset: offset.
		dt := DateAndTime 
			year: yr month: mo day: day 
			hour: hour minute: min second: sec nanoSecond: nano 
			offset: offset.
		self assert: dt julianDayNumber equals: lxdt julianDayNumber].
! !

!LXDateAndTimeConversionTest methodsFor: 'testing' stamp: 'dtl 4/9/2014 22:28'!
testMinute
	"(self selector: #testMinute) debug"

	| day dt hour lxdt min mo nano offset sec |
	mo := 12.
	day := 2.
	hour := 2.
	min := 3.
	sec := 4.
	nano := 5.
	offset := 6 hours.
	-2000 to: 5000 do: [:yr |
		lxdt := LXDateAndTime 
			year: yr month: mo day: day 
			hour: hour minute: min second: sec nanoSecond: nano 
			offset: offset.
		dt := DateAndTime 
			year: yr month: mo day: day 
			hour: hour minute: min second: sec nanoSecond: nano 
			offset: offset.
		self assert: dt minute equals: lxdt minute].
! !

!LXDateAndTimeConversionTest methodsFor: 'testing' stamp: 'dtl 4/5/2014 09:57'!
testMonth

	| day dt hour lxdt min mo nano offset sec yr |
	yr := 1969.
	mo := 12.
	day := 2.
	hour := 2.
	min := 3.
	sec := 4.
	nano := 5.
	offset := 6 hours.
	lxdt := LXDateAndTime 
			year: yr month: mo day: day 
			hour: hour minute: min second: sec nanoSecond: nano 
			offset: offset.
	dt := DateAndTime 
			year: yr month: mo day: day 
			hour: hour minute: min second: sec nanoSecond: nano 
			offset: offset.
	self assert: 12 equals: dt month.
	self assert: 12 equals: lxdt month.

	(2105 to: -1105 by: 20)
		do: [:year |
			1 to: 12 do: [:m |
				lxdt := LXDateAndTime 
					year: year month: m day: day 
					hour: hour minute: min second: sec nanoSecond: nano 
					offset: offset.
				dt := DateAndTime 
					year: year month: m day: day 
					hour: hour minute: min second: sec nanoSecond: nano 
					offset: offset.
				self assert: m equals: dt month.
				self assert: m equals: lxdt month]]
! !

!LXDateAndTimeConversionTest methodsFor: 'testing' stamp: 'dtl 4/5/2014 09:52'!
testNanoSecond

	| day dt hour lxdt min mo nano offset sec yr |
	yr := 1969.
	mo := 12.
	day := 2.
	hour := 2.
	min := 3.
	sec := 4.
	nano := 5.
	offset := 6 hours.
	lxdt := LXDateAndTime 
			year: yr month: mo day: day 
			hour: hour minute: min second: sec nanoSecond: nano 
			offset: offset.
	dt := DateAndTime 
			year: yr month: mo day: day 
			hour: hour minute: min second: sec nanoSecond: nano 
			offset: offset.
	self assert: 5 equals: dt nanoSecond.
	self assert: 5 equals: lxdt nanoSecond.

	(2105 to: -1105 by: 20)
		do: [:year |
			lxdt := LXDateAndTime 
				year: year month: mo day: day 
				hour: hour minute: min second: sec nanoSecond: nano 
				offset: offset.
			dt := DateAndTime 
				year: year month: mo day: day 
				hour: hour minute: min second: sec nanoSecond: nano 
				offset: offset.
			self assert: 5 equals: dt nanoSecond.
			self assert: 5 equals: lxdt nanoSecond]

! !

!LXDateAndTimeConversionTest methodsFor: 'testing' stamp: 'dtl 4/8/2014 20:47'!
testSecond
	"(self selector: #testSecond) debug"

	| day dt hour lxdt min mo nano offset sec |
	mo := 12.
	day := 2.
	hour := 2.
	min := 3.
	sec := 4.
	nano := 5.
	offset := 6 hours.
	-2000 to: 5000 do: [:yr |
		lxdt := LXDateAndTime 
			year: yr month: mo day: day 
			hour: hour minute: min second: sec nanoSecond: nano 
			offset: offset.
		dt := DateAndTime 
			year: yr month: mo day: day 
			hour: hour minute: min second: sec nanoSecond: nano 
			offset: offset.
		self assert: dt second equals: lxdt second].
! !

!LXDateAndTimeConversionTest methodsFor: 'testing' stamp: 'dtl 4/5/2014 09:54'!
testYear

	| day dt hour lxdt min mo nano offset sec yr |
	yr := 1969.
	mo := 12.
	day := 2.
	hour := 2.
	min := 3.
	sec := 4.
	nano := 5.
	offset := 6 hours.
	lxdt := LXDateAndTime 
			year: yr month: mo day: day 
			hour: hour minute: min second: sec nanoSecond: nano 
			offset: offset.
	dt := DateAndTime 
			year: yr month: mo day: day 
			hour: hour minute: min second: sec nanoSecond: nano 
			offset: offset.
	self assert: 1969 equals: dt year.
	self assert: 1969 equals: lxdt year.

	(2105 to: -1105 by: 20)
		do: [:year |
			lxdt := LXDateAndTime 
				year: year month: mo day: day 
				hour: hour minute: min second: sec nanoSecond: nano 
				offset: offset.
			dt := DateAndTime 
				year: year month: mo day: day 
				hour: hour minute: min second: sec nanoSecond: nano 
				offset: offset.
			self assert: year equals: dt year.
			self assert: year equals: lxdt year]

! !


!LXDateAndTimeEpochTest methodsFor: 'running' stamp: 'dtl 3/23/2014 12:30'!
setUp
     localTimeZoneToRestore := LXDateAndTime localTimeZone.
	aDateAndTime :=  LXDateAndTime localTimeZone: TimeZone default; epoch.
	aTimeZone := TimeZone offset: (Duration minutes: 135) name: 'Epoch Test Time Zone' abbreviation: 'ETZ'.
	aDuration := Duration days: 1 hours: 2 minutes: 3 seconds: 4 nanoSeconds: 5 ! !

!LXDateAndTimeEpochTest methodsFor: 'running' stamp: 'tlk 1/2/2004 11:04'!
tearDown
     LXDateAndTime localTimeZone: localTimeZoneToRestore.
     "wish I could remove the time zones I added earlier, tut there is no method for that"
! !

!LXDateAndTimeEpochTest methodsFor: 'testing' stamp: 'tlk 1/4/2004 14:01'!
testAsDate
	self assert: aDateAndTime asDate =   'January 1, 1901' asDate.
! !

!LXDateAndTimeEpochTest methodsFor: 'testing' stamp: 'tlk 1/1/2004 13:31'!
testAsDateAndTime
	self assert: aDateAndTime asLXDateAndTime =  aDateAndTime
	
! !

!LXDateAndTimeEpochTest methodsFor: 'testing' stamp: 'tlk 1/1/2004 13:34'!
testAsDuration
	self assert: aDateAndTime asDuration =  0 asDuration
	
! !

!LXDateAndTimeEpochTest methodsFor: 'testing' stamp: 'tlk 1/2/2004 11:06'!
testAsLocal
	self assert: aDateAndTime asLocal =  aDateAndTime.
	self assert: aDateAndTime asLocal = (aDateAndTime utcOffset: aDateAndTime class localOffset)
	
! !

!LXDateAndTimeEpochTest methodsFor: 'testing' stamp: 'tlk 1/4/2004 09:27'!
testAsMonth
	self assert: aDateAndTime asMonth = (Month month: 'January' year: 1901). 
! !

!LXDateAndTimeEpochTest methodsFor: 'testing' stamp: 'tlk 1/1/2004 13:59'!
testAsNanoSeconds
	self assert: aDateAndTime asNanoSeconds =  0 asDuration asNanoSeconds
	
! !

!LXDateAndTimeEpochTest methodsFor: 'testing' stamp: 'tlk 1/1/2004 14:01'!
testAsSeconds
	self assert: aDateAndTime asSeconds =  0 asDuration asSeconds
	
! !

!LXDateAndTimeEpochTest methodsFor: 'testing' stamp: 'tlk 1/4/2004 09:32'!
testAsTime
	self assert: aDateAndTime asTime =  Time midnight.
! !

!LXDateAndTimeEpochTest methodsFor: 'testing' stamp: 'tlk 1/1/2004 14:51'!
testAsTimeStamp
	self assert: aDateAndTime asTimeStamp =  TimeStamp new.
! !

!LXDateAndTimeEpochTest methodsFor: 'testing' stamp: 'tlk 1/2/2004 11:07'!
testAsUTC
	self assert: aDateAndTime asUTC =  aDateAndTime
          ! !

!LXDateAndTimeEpochTest methodsFor: 'testing' stamp: 'brp 1/16/2004 13:43'!
testAsWeek
	self assert: aDateAndTime asWeek = (Week starting: '12-31-1900' asDate). 
! !

!LXDateAndTimeEpochTest methodsFor: 'testing' stamp: 'tlk 1/4/2004 09:43'!
testAsYear
	self assert: aDateAndTime asYear =   (Year starting: '01-01-1901' asDate). 
! !

!LXDateAndTimeEpochTest methodsFor: 'testing' stamp: 'tlk 1/1/2004 12:28'!
testCurrent
	self deny: aDateAndTime =  (LXDateAndTime current).
! !

!LXDateAndTimeEpochTest methodsFor: 'testing' stamp: 'tlk 1/4/2004 09:46'!
testDateTime
	self assert: aDateAndTime =  (LXDateAndTime date: '01-01-1901' asDate time: '00:00:00' asTime)
! !

!LXDateAndTimeEpochTest methodsFor: 'testing' stamp: 'tlk 1/4/2004 14:01'!
testDay
	self assert: aDateAndTime day =   LXDateAndTime new day
! !

!LXDateAndTimeEpochTest methodsFor: 'testing' stamp: 'tlk 1/2/2004 11:08'!
testDayMonthYearDo
	|iterations|
	iterations := 0.
	self assert: (aDateAndTime dayMonthYearDo: [:eachDay :eachMonth :eachYear |  iterations := iterations + 1])  = 1.
	self assert: (aDateAndTime dayMonthYearDo: [:eachDay :eachMonth :eachYear |  eachYear])  = 1901.
	self assert: (aDateAndTime dayMonthYearDo: [:eachDay :eachMonth :eachYear |  eachMonth]) = 1.
	self assert: (aDateAndTime dayMonthYearDo: [:eachDay :eachMonth :eachYear |  eachDay]) = 1.
! !

!LXDateAndTimeEpochTest methodsFor: 'testing' stamp: 'tlk 1/1/2004 15:45'!
testDayOfMonth
	self assert: aDateAndTime dayOfMonth  = 1.
! !

!LXDateAndTimeEpochTest methodsFor: 'testing' stamp: 'tlk 1/4/2004 10:47'!
testDayOfWeek
	self assert: aDateAndTime dayOfWeek  = 3.
	self assert: aDateAndTime dayOfWeekAbbreviation = 'Tue'.
	self assert: aDateAndTime dayOfWeekName = 'Tuesday'.
! !

!LXDateAndTimeEpochTest methodsFor: 'testing' stamp: 'tlk 1/4/2004 14:01'!
testDayOfYear
	self assert: aDateAndTime dayOfYear  = 1.
! !

!LXDateAndTimeEpochTest methodsFor: 'testing' stamp: 'tlk 1/4/2004 14:02'!
testDaysInMonth
	self assert: aDateAndTime daysInMonth  = 31.
! !

!LXDateAndTimeEpochTest methodsFor: 'testing' stamp: 'tlk 1/4/2004 14:02'!
testDaysInYear
	self assert: aDateAndTime daysInYear  = 365.
! !

!LXDateAndTimeEpochTest methodsFor: 'testing' stamp: 'tlk 1/4/2004 14:02'!
testDaysLeftInYear
	self assert: aDateAndTime daysLeftInYear  = 364.
! !

!LXDateAndTimeEpochTest methodsFor: 'testing' stamp: 'tlk 1/1/2004 16:24'!
testDuration
	self assert: aDateAndTime duration  = 0 asDuration.
! !

!LXDateAndTimeEpochTest methodsFor: 'testing' stamp: 'cmm 6/14/2010 17:19'!
testEpoch
	self assert: aDateAndTime =  '1901-01-01T00:00:00+00:00' asLXDateAndTime
! !

!LXDateAndTimeEpochTest methodsFor: 'testing' stamp: 'tlk 1/4/2004 10:44'!
testFirstDayOfMonth
	self assert: aDateAndTime firstDayOfMonth =   1
! !

!LXDateAndTimeEpochTest methodsFor: 'testing' stamp: 'tlk 1/1/2004 12:25'!
testFromSeconds
	self assert: aDateAndTime =  (LXDateAndTime fromSeconds: 0).
! !

!LXDateAndTimeEpochTest methodsFor: 'testing' stamp: 'tlk 1/1/2004 12:26'!
testFromString
	self assert: aDateAndTime =  (LXDateAndTime fromString: ' 1901-01-01T00:00:00+00:00').
	self assert: aDateAndTime =  (LXDateAndTime fromString: ' 1901-01-01T00:00:00').
	self assert: aDateAndTime =  (LXDateAndTime fromString: ' 1901-01-01T00:00').
	self assert: aDateAndTime =  (LXDateAndTime fromString: ' 1901-01-01T00:00:00+00:00').
! !

!LXDateAndTimeEpochTest methodsFor: 'testing' stamp: 'dtl 3/23/2014 12:30'!
testHash
	self assert: aDateAndTime hash =    LXDateAndTime new hash.
	self assert: aDateAndTime hash =     112557138
! !

!LXDateAndTimeEpochTest methodsFor: 'testing' stamp: 'tlk 1/1/2004 16:59'!
testHour
	self assert: aDateAndTime hour =    aDateAndTime hour24.
	self assert: aDateAndTime hour =    0.
	self assert: aDateAndTime hour =    aDateAndTime hours
! !

!LXDateAndTimeEpochTest methodsFor: 'testing' stamp: 'brp 3/12/2004 15:21'!
testHour12
	self assert: aDateAndTime hour12  = LXDateAndTime new hour12.
	self assert: aDateAndTime hour12  = 12
! !

!LXDateAndTimeEpochTest methodsFor: 'testing' stamp: 'tlk 1/4/2004 14:02'!
testIsLeapYear
	self deny: aDateAndTime isLeapYear
! !

!LXDateAndTimeEpochTest methodsFor: 'testing' stamp: 'tlk 1/1/2004 17:18'!
testJulianDayNumber
	self assert: aDateAndTime =  (LXDateAndTime julianDayNumber: 2415386).
	self assert: aDateAndTime julianDayNumber = 2415386.! !

!LXDateAndTimeEpochTest methodsFor: 'testing' stamp: 'tlk 1/1/2004 13:20'!
testLessThan
	self assert: aDateAndTime  < (aDateAndTime + '1:00:00:00').
	self assert: aDateAndTime + -1 < aDateAndTime.
	! !

!LXDateAndTimeEpochTest methodsFor: 'testing' stamp: 'tlk 1/4/2004 10:40'!
testMeridianAbbreviation
	self assert: aDateAndTime meridianAbbreviation = 'AM'.
	! !

!LXDateAndTimeEpochTest methodsFor: 'testing' stamp: 'tlk 1/4/2004 12:37'!
testMiddleOf
	self assert: (aDateAndTime middleOf: '2:00:00:00' asDuration) = 
	 (Timespan starting: '12-31-1900' asDate duration: 2 days).
	
! !

!LXDateAndTimeEpochTest methodsFor: 'testing' stamp: 'tlk 1/1/2004 17:39'!
testMidnight
	self assert: aDateAndTime midnight =  aDateAndTime
! !

!LXDateAndTimeEpochTest methodsFor: 'testing' stamp: 'dtl 3/23/2014 12:30'!
testMinus
	self assert: aDateAndTime - aDateAndTime =  '0:00:00:00' asDuration.
	self assert: aDateAndTime - '0:00:00:00' asDuration = aDateAndTime.
	self assert: aDateAndTime - aDuration =  (LXDateAndTime year: 1900 month: 12 day: 30 hour: 21 minute: 56 second: 55 nanoSecond: 999999995 offset: 0 hours ).
	" I believe this Failure is a bug in the nanosecond part of (LXDateAndTime >> year:month:day:hour:minute:second:nanoSecond:offset:)" ! !

!LXDateAndTimeEpochTest methodsFor: 'testing' stamp: 'tlk 1/1/2004 19:35'!
testMinute
	self assert: aDateAndTime minute =  0
! !

!LXDateAndTimeEpochTest methodsFor: 'testing' stamp: 'brp 1/16/2004 13:41'!
testMinutes
	self assert: aDateAndTime minutes = 0
! !

!LXDateAndTimeEpochTest methodsFor: 'testing' stamp: 'tlk 1/1/2004 19:46'!
testMonth
	self assert: aDateAndTime month  = 1.
	self assert: aDateAndTime monthAbbreviation = 'Jan'.
	self assert: aDateAndTime monthName = 'January'.
	self assert: aDateAndTime monthIndex = 1.! !

!LXDateAndTimeEpochTest methodsFor: 'testing' stamp: 'tlk 1/1/2004 19:47'!
testNanoSecond
	self assert: aDateAndTime nanoSecond =  0
! !

!LXDateAndTimeEpochTest methodsFor: 'testing' stamp: 'tlk 1/1/2004 12:27'!
testNew
	self assert: aDateAndTime =  (LXDateAndTime new).
! !

!LXDateAndTimeEpochTest methodsFor: 'testing' stamp: 'cmm 6/14/2010 17:19'!
testNoon
	self assert: aDateAndTime noon = '1901-01-01T12:00:00+00:00' asLXDateAndTime! !

!LXDateAndTimeEpochTest methodsFor: 'testing' stamp: 'tlk 1/1/2004 12:28'!
testNow
	self deny: aDateAndTime =  (LXDateAndTime now).
! !

!LXDateAndTimeEpochTest methodsFor: 'testing' stamp: 'cmm 6/14/2010 17:20'!
testOffset
	self assert: aDateAndTime offset =  '0:00:00:00' asDuration.
     self assert: (aDateAndTime offset: '0:12:00:00') =  '1901-01-01T00:00:00+12:00' asLXDateAndTime! !

!LXDateAndTimeEpochTest methodsFor: 'testing' stamp: 'tlk 1/4/2004 11:03'!
testPlus
	self assert: aDateAndTime + '0:00:00:00' = aDateAndTime.
	self assert: aDateAndTime + 0 = aDateAndTime.
	self assert: aDateAndTime + aDuration = (LXDateAndTime year: 1901 month: 1 day: 2 hour: 2 minute: 3 second: 4 nanoSecond: 5 offset: 0 hours )
	" I believe this is a bug in the nanosecond part of (DateAndTime >> year:month:day:hour:minute:second:nanoSecond:offset:)"
	
! !

!LXDateAndTimeEpochTest methodsFor: 'testing' stamp: 'nice 2/23/2013 20:23'!
testPrintOn
	| ref ws |
	ref := '1901-01-01T00:00:00+00:00'.
	ws := '' writeStream.
	aDateAndTime printOn: ws.
	self assert: ws contents = ref.
	ref  := 'a TimeZone(ETZ)'.
	ws := '' writeStream.
	aTimeZone printOn:  ws.
	self assert: ws contents = ref! !

!LXDateAndTimeEpochTest methodsFor: 'testing' stamp: 'tlk 1/1/2004 20:22'!
testSecond
	self assert: aDateAndTime second =  0
! !

!LXDateAndTimeEpochTest methodsFor: 'testing' stamp: 'tlk 1/1/2004 20:22'!
testSeconds
	self assert: aDateAndTime seconds =  0
! !

!LXDateAndTimeEpochTest methodsFor: 'testing' stamp: 'tlk 1/1/2004 20:25'!
testTicks
	self assert: aDateAndTime ticks =  (LXDateAndTime julianDayNumber: 2415386) ticks.
	self assert: aDateAndTime ticks = #(2415386 0 0)! !

!LXDateAndTimeEpochTest methodsFor: 'testing' stamp: 'tlk 1/1/2004 20:31'!
testTicksOffset
	self assert: aDateAndTime =  (aDateAndTime ticks:  #(2415386 0 0) offset: LXDateAndTime localOffset).
! !

!LXDateAndTimeEpochTest methodsFor: 'testing' stamp: 'tlk 1/3/2004 11:42'!
testTo
	self assert: (aDateAndTime to: aDateAndTime) = (LXDateAndTime new to: LXDateAndTime new) 
	"MessageNotUnderstood: UndefinedObject>>starting:ending:  where UndefinedObject is Timespan "! !

!LXDateAndTimeEpochTest methodsFor: 'testing' stamp: 'tlk 1/3/2004 11:43'!
testToBy
	self assert: (aDateAndTime to: aDateAndTime + 10 days by: 5 days) = 
				(LXDateAndTime new to: LXDateAndTime new + 10 days by: 5 days ) 
	"MessageNotUnderstood: UndefinedObject>>starting:ending:  where UndefinedObject is Timespan "! !

!LXDateAndTimeEpochTest methodsFor: 'testing' stamp: 'tlk 1/1/2004 20:53'!
testToByDo
	"self assert: (aDateAndTime to: aDateAndTime + 10 days by: 5 days do: []) =  "
	"MessageNotUnderstood: UndefinedObject>>starting:ending:  where UndefinedObject is Timespan "! !

!LXDateAndTimeEpochTest methodsFor: 'testing' stamp: 'tlk 1/1/2004 17:35'!
testToday
	self deny: aDateAndTime =  (LXDateAndTime today).
! !

!LXDateAndTimeEpochTest methodsFor: 'testing' stamp: 'tlk 1/3/2004 11:45'!
testTommorrow
	self assert: (LXDateAndTime today + 24 hours) =  (LXDateAndTime tomorrow).
	self deny: aDateAndTime =  (LXDateAndTime tomorrow).
     "MessageNotUnderstood: Date class>>starting:"! !

!LXDateAndTimeEpochTest methodsFor: 'testing' stamp: 'cmm 6/14/2010 17:20'!
testUtcOffset
     self assert: (aDateAndTime utcOffset: '0:12:00:00') =  '1901-01-01T12:00:00+12:00' asLXDateAndTime! !

!LXDateAndTimeEpochTest methodsFor: 'testing' stamp: 'tlk 1/1/2004 21:00'!
testYear
	self assert: aDateAndTime year = 1901.
	! !

!LXDateAndTimeEpochTest methodsFor: 'testing' stamp: 'tlk 1/1/2004 12:30'!
testYearDay
	self assert: aDateAndTime =  (LXDateAndTime year: 1901 day: 1).
! !

!LXDateAndTimeEpochTest methodsFor: 'testing' stamp: 'tlk 1/1/2004 12:31'!
testYearDayHourMinuteSecond
	self assert: aDateAndTime =  (LXDateAndTime year: 1901 day: 1 hour: 0 minute: 0 second: 0).
! !

!LXDateAndTimeEpochTest methodsFor: 'testing' stamp: 'tlk 1/1/2004 12:31'!
testYearMonthDay
	self assert: aDateAndTime =  (LXDateAndTime year: 1901 month: 1 day: 1).
! !

!LXDateAndTimeEpochTest methodsFor: 'testing' stamp: 'tlk 1/1/2004 12:31'!
testYearMonthDayHourMinuteSecond
	self assert: aDateAndTime =  (LXDateAndTime year: 1901 month: 1 day: 1 hour: 0 minute: 0 second: 0).
! !

!LXDateAndTimeEpochTest methodsFor: 'testing' stamp: 'tlk 1/4/2004 12:23'!
testYearMonthDayHourMinuteSecondNanosSecondOffset
	self assert: aDateAndTime =  (LXDateAndTime year: 1901 month: 1 day: 1 hour: 0 minute: 0 second: 0 nanoSecond: 0 offset:0 hours ).
	self assert: ((LXDateAndTime year: 1 month: 1 day: 1 hour: 0 minute: 0 second: 0 nanoSecond: 0 offset: 0 hours ) +
				(Duration days: 1 hours: 2 minutes: 3 seconds: 4  nanoSeconds: 5) ) =  	
				(LXDateAndTime year: 1 month: 1 day: 2 hour: 2 minute: 3 second: 4 nanoSecond: 5 offset: 0 hours ) 
	" I believe this is a bug in the nanosecond part of (DateAndTime >> year:month:day:hour:minute:second:nanoSecond:offset:)"" I believe this is a bug in the nanosecond part of (DateAndTime >> year:month:day:hour:minute:second:nanoSecond:offset:)"   
! !

!LXDateAndTimeEpochTest methodsFor: 'testing' stamp: 'tlk 1/4/2004 09:47'!
testYesterday
	self deny: aDateAndTime =  (LXDateAndTime yesterday).
! !

!LXDateAndTimeEpochTest methodsFor: 'testing' stamp: 'nk 3/12/2004 11:26'!
testtimeZone
	self assert: aDateAndTime timeZoneName	= 'Universal Time'.
	self assert: aDateAndTime timeZoneAbbreviation	=  'UTC'
! !


!LXDateAndTimeLeapTest methodsFor: 'running' stamp: 'nk 3/12/2004 11:00'!
setUp
	localTimeZoneToRestore := LXDateAndTime localTimeZone.
	LXDateAndTime localTimeZone: TimeZone default.
	aDateAndTime := (LXDateAndTime year: 2004 month: 2 day: 29 hour: 13 minute: 33 second: 0 offset: 2 hours).
	aTimeZone := TimeZone default.
	aDuration := Duration days: 0 hours: 13 minutes: 33 seconds: 0 nanoSeconds: 0
! !

!LXDateAndTimeLeapTest methodsFor: 'running' stamp: 'tlk 1/2/2004 21:30'!
tearDown
     LXDateAndTime localTimeZone: localTimeZoneToRestore.
     "wish I could remove the time zones I added earlier, tut there is no method for that"
! !

!LXDateAndTimeLeapTest methodsFor: 'testing' stamp: 'cmm 2/2/2012 21:48'!
testAsDate
	self assert: (aDateAndTime offset: LXDateAndTime localTimeZone offset) asDate = 'February 29, 2004' asDate! !

!LXDateAndTimeLeapTest methodsFor: 'testing' stamp: 'tlk 1/2/2004 21:55'!
testAsDuration
	self assert: aDateAndTime asDuration =  aDuration
	
! !

!LXDateAndTimeLeapTest methodsFor: 'testing' stamp: 'tlk 1/4/2004 14:00'!
testAsLocal
	self assert: aDateAndTime asLocal =  aDateAndTime.
	self assert: aDateAndTime asLocal = (aDateAndTime utcOffset: aDateAndTime class localOffset)
	
! !

!LXDateAndTimeLeapTest methodsFor: 'testing' stamp: 'cmm 2/2/2012 22:51'!
testAsMonth
	self assert:
		(aDateAndTime offset: Month defaultOffset) asMonth =
			(Month
				month: 'February'
				year: 2004)! !

!LXDateAndTimeLeapTest methodsFor: 'testing' stamp: 'tlk 1/2/2004 21:59'!
testAsNanoSeconds
	self assert: aDateAndTime asNanoSeconds =  aDuration asNanoSeconds.
	self assert: aDateAndTime asNanoSeconds = 48780000000000
	
! !

!LXDateAndTimeLeapTest methodsFor: 'testing' stamp: 'cmm 2/2/2012 22:53'!
testAsSeconds
	self assert: aDuration asSeconds = 48780.
	self assert: aDateAndTime asSeconds = 3255514380! !

!LXDateAndTimeLeapTest methodsFor: 'testing' stamp: 'tlk 1/4/2004 12:26'!
testAsTime
	self assert: aDateAndTime asTime = (Time hour: 13 minute: 33 second: 0)
! !

!LXDateAndTimeLeapTest methodsFor: 'testing' stamp: 'tlk 1/4/2004 13:31'!
testAsTimeStamp
	self assert: aDateAndTime asTimeStamp =  ((TimeStamp readFrom: '2-29-2004 1:33 pm' readStream) offset: 2 hours).
! !

!LXDateAndTimeLeapTest methodsFor: 'testing' stamp: 'tlk 1/4/2004 13:59'!
testAsUTC
	self assert: aDateAndTime asUTC =  aDateAndTime
          ! !

!LXDateAndTimeLeapTest methodsFor: 'testing' stamp: 'dtl 3/23/2014 12:30'!
testAsWeek
	self assert: (aDateAndTime offset: LXDateAndTime localTimeZone offset) asWeek = (Week starting: '02-29-2004' asDate)! !

!LXDateAndTimeLeapTest methodsFor: 'testing' stamp: 'cmm 2/2/2012 21:49'!
testAsYear
	| year |
	year := (aDateAndTime offset: LXDateAndTime localTimeZone offset) asYear.
	self assert: year = (Year starting: '02-29-2004' asDate).
	self deny: year = (Year starting: '01-01-2004' asDate)! !

!LXDateAndTimeLeapTest methodsFor: 'testing' stamp: 'tlk 1/4/2004 13:23'!
testDay
	self assert: aDateAndTime day =   60. 
	self deny: aDateAndTime day =   29 ! !

!LXDateAndTimeLeapTest methodsFor: 'testing' stamp: 'tlk 1/2/2004 22:16'!
testDayMonthYearDo
	self assert: (aDateAndTime dayMonthYearDo: [:eachDay :eachMonth :eachYear |  eachYear])  = 2004.
	self assert: (aDateAndTime dayMonthYearDo: [:eachDay :eachMonth :eachYear |  eachMonth]) = 2.
	self assert: (aDateAndTime dayMonthYearDo: [:eachDay :eachMonth :eachYear |  eachDay]) = 29.
! !

!LXDateAndTimeLeapTest methodsFor: 'testing' stamp: 'tlk 1/2/2004 22:17'!
testDayOfMonth
	self assert: aDateAndTime dayOfMonth  = 29.
! !

!LXDateAndTimeLeapTest methodsFor: 'testing' stamp: 'tlk 1/4/2004 13:34'!
testDayOfWeek
	self assert: aDateAndTime dayOfWeek  = 1.
	self assert: aDateAndTime dayOfWeekAbbreviation = 'Sun'.
	self assert: aDateAndTime dayOfWeekName = 'Sunday'.
! !

!LXDateAndTimeLeapTest methodsFor: 'testing' stamp: 'tlk 1/4/2004 13:59'!
testDayOfYear
	self assert: aDateAndTime dayOfYear  = 60.
! !

!LXDateAndTimeLeapTest methodsFor: 'testing' stamp: 'tlk 1/4/2004 13:58'!
testDaysInMonth
	self assert: aDateAndTime daysInMonth  = 29.
! !

!LXDateAndTimeLeapTest methodsFor: 'testing' stamp: 'tlk 1/4/2004 13:58'!
testDaysInYear
	self assert: aDateAndTime daysInYear  = 366.
! !

!LXDateAndTimeLeapTest methodsFor: 'testing' stamp: 'tlk 1/4/2004 13:58'!
testDaysLeftInYear
	self assert: aDateAndTime daysLeftInYear  = 306.
! !

!LXDateAndTimeLeapTest methodsFor: 'testing' stamp: 'tlk 1/4/2004 13:38'!
testFirstDayOfMonth
	self deny: aDateAndTime firstDayOfMonth =  1.
	self assert: aDateAndTime firstDayOfMonth = 32
! !

!LXDateAndTimeLeapTest methodsFor: 'testing' stamp: 'tlk 1/3/2004 10:43'!
testFromString
	self assert: aDateAndTime =  (LXDateAndTime fromString: ' 2004-02-29T13:33:00+02:00').
! !

!LXDateAndTimeLeapTest methodsFor: 'testing' stamp: 'pad 5/10/2012 17:12'!
testHash
	self assert: aDateAndTime hash =      16256473
! !

!LXDateAndTimeLeapTest methodsFor: 'testing' stamp: 'tlk 1/3/2004 10:48'!
testHour
	self assert: aDateAndTime hour =    aDateAndTime hour24.
	self assert: aDateAndTime hour =    13.
	self assert: aDateAndTime hour =    aDateAndTime hours
! !

!LXDateAndTimeLeapTest methodsFor: 'testing' stamp: 'brp 3/12/2004 15:19'!
testHour12
	self assert: aDateAndTime hour12  =   1.
! !

!LXDateAndTimeLeapTest methodsFor: 'testing' stamp: 'tlk 1/4/2004 13:35'!
testIsLeapYear
	self assert: aDateAndTime isLeapYear
! !

!LXDateAndTimeLeapTest methodsFor: 'testing' stamp: 'tlk 1/2/2004 21:30'!
testLessThan
	self assert: aDateAndTime  < (aDateAndTime + '1:00:00:00').
	self assert: aDateAndTime + -1 < aDateAndTime.
	! !

!LXDateAndTimeLeapTest methodsFor: 'testing' stamp: 'tlk 1/4/2004 10:42'!
testMeridianAbbreviation
	self assert: aDateAndTime meridianAbbreviation = 'PM'.
	! !

!LXDateAndTimeLeapTest methodsFor: 'testing' stamp: 'tlk 1/4/2004 13:12'!
testMiddleOf
	self assert: (aDateAndTime middleOf: aDuration)  = 
	 (Timespan starting: (LXDateAndTime year: 2004 month: 2 day: 29 hour: 6 minute: 46 second: 30 offset: 2 hours)
	duration: (Duration days: 0 hours: 13 minutes: 33 seconds: 0 nanoSeconds: 0 ))
	! !

!LXDateAndTimeLeapTest methodsFor: 'testing' stamp: 'cmm 2/2/2012 21:51'!
testMidnight
	| midnight |
	midnight := (aDateAndTime offset: LXDateAndTime localTimeZone offset) midnight.
	self assert: midnight = '2004-02-29T00:00:00+00:00' asLXDateAndTime.
	self deny: midnight = '2004-02-29T00:00:00+02:00' asLXDateAndTime! !

!LXDateAndTimeLeapTest methodsFor: 'testing' stamp: 'tlk 1/3/2004 11:00'!
testMinute
	self assert: aDateAndTime minute =  33
! !

!LXDateAndTimeLeapTest methodsFor: 'testing' stamp: 'brp 1/16/2004 13:44'!
testMinutes
	self assert: aDateAndTime minutes = 33
! !

!LXDateAndTimeLeapTest methodsFor: 'testing' stamp: 'tlk 1/3/2004 11:02'!
testMonth
	self assert: aDateAndTime month  = 2.
	self assert: aDateAndTime monthAbbreviation = 'Feb'.
	self assert: aDateAndTime monthName = 'February'.
	self assert: aDateAndTime monthIndex = 2.! !

!LXDateAndTimeLeapTest methodsFor: 'testing' stamp: 'cmm 4/28/2011 22:28'!
testMonthParsing
	self assert:
		(Month readFrom: 'Feb 2011' readStream) =
			(Month
				month: 2
				year: 2011)! !

!LXDateAndTimeLeapTest methodsFor: 'testing' stamp: 'tlk 1/2/2004 21:30'!
testNanoSecond
	self assert: aDateAndTime nanoSecond =  0
! !

!LXDateAndTimeLeapTest methodsFor: 'testing' stamp: 'cmm 6/14/2010 17:21'!
testNoon
	self assert: aDateAndTime noon =  '2004-02-29T12:00:00+00:00' asLXDateAndTime! !

!LXDateAndTimeLeapTest methodsFor: 'testing' stamp: 'cmm 6/14/2010 17:21'!
testOffset
	self assert: aDateAndTime offset =  '0:02:00:00' asDuration.
     self assert: (aDateAndTime offset: '0:12:00:00') =  '2004-02-29T13:33:00+12:00' asLXDateAndTime! !

!LXDateAndTimeLeapTest methodsFor: 'testing' stamp: 'nice 2/23/2013 20:32'!
testPrintOn
	| ref ws |
	ref := '2004-02-29T13:33:00+02:00'.
	ws := '' writeStream.
	aDateAndTime printOn: ws.
	self assert: ws contents = ref.
	ref  := 'a TimeZone(UTC)'.
	ws := '' writeStream.
	aTimeZone printOn:  ws.
	self assert: ws contents = ref	! !

!LXDateAndTimeLeapTest methodsFor: 'testing' stamp: 'tlk 1/2/2004 21:30'!
testSecond
	self assert: aDateAndTime second =  0
! !

!LXDateAndTimeLeapTest methodsFor: 'testing' stamp: 'tlk 1/2/2004 21:30'!
testSeconds
	self assert: aDateAndTime seconds =  0
! !

!LXDateAndTimeLeapTest methodsFor: 'testing' stamp: 'tlk 1/3/2004 11:12'!
testTicks
	self assert: aDateAndTime ticks =  ((LXDateAndTime julianDayNumber: 2453065) + 48780 seconds) ticks.
	self assert: aDateAndTime ticks =  #(2453065 48780 0)! !

!LXDateAndTimeLeapTest methodsFor: 'testing' stamp: 'tlk 1/4/2004 13:52'!
testTicksOffset
	self assert: aDateAndTime =  (aDateAndTime ticks:  #(2453065 48780 0) offset: LXDateAndTime localOffset).
! !

!LXDateAndTimeLeapTest methodsFor: 'testing' stamp: 'cmm 6/14/2010 17:22'!
testUtcOffset
     self assert: (aDateAndTime utcOffset: '0:02:00:00') =  '2004-02-29T13:33:00+02:00' asLXDateAndTime! !

!LXDateAndTimeLeapTest methodsFor: 'testing' stamp: 'tlk 1/3/2004 11:17'!
testYear
	self assert: aDateAndTime year = 2004.
	! !

!LXDateAndTimeLeapTest methodsFor: 'testing' stamp: 'tlk 1/3/2004 11:21'!
testYearDayHourMinuteSecond
	self assert: aDateAndTime =  ((LXDateAndTime year: 2004 day: 60 hour: 13 minute: 33 second: 0) offset: 2 hours).
! !

!LXDateAndTimeLeapTest methodsFor: 'testing' stamp: 'tlk 1/3/2004 11:23'!
testYearMonthDayHourMinuteSecond
	self assert: aDateAndTime =  ((LXDateAndTime year: 2004 month: 2 day: 29 hour: 13 minute: 33 second: 0) offset: 2 hours).
! !

!LXDateAndTimeLeapTest methodsFor: 'testing' stamp: 'nk 3/12/2004 11:26'!
testtimeZone
	self assert: aDateAndTime timeZoneName	= 'Universal Time'.
	self assert: aDateAndTime timeZoneAbbreviation	=  'UTC'
! !


!LXDateAndTimeTest methodsFor: 'Coverage' stamp: 'brp 9/25/2003 09:25'!
classToBeTested
	^ LXDateAndTime
! !

!LXDateAndTimeTest methodsFor: 'Coverage' stamp: 'brp 9/25/2003 09:25'!
selectorsToBeIgnored
	| private | 
	private := #( #printOn: ).
	^ super selectorsToBeIgnored, private
! !

!LXDateAndTimeTest methodsFor: 'Tests' stamp: 'brp 1/7/2004 17:00'!
testArithmeticAcrossDateBoundary
	| t1 t2 |
	t1 := '2004-01-07T11:55:00+00:00' asLXDateAndTime. 
	t2 := t1 - ( (42900+1) seconds).  
	self 
		assert: t2 = ('2004-01-06T23:59:59+00:00' asLXDateAndTime)
		
! !

!LXDateAndTimeTest methodsFor: 'Tests' stamp: 'dtl 11/7/2004 13:00'!
testDateTimeDenotation1
  "LXDateAndTimeTest new testDateTimeDenotation1"
	
	 " Detroit is 5 hours behind UTC, this offset to UTC is therefore written with a minus sign. This example tests the correct interpretation of the DateAndTime denotation. "
	| twoPmInLondon twoPmUTCInLocalTimeOfDetroit nineAmInDetroit |
	twoPmInLondon := LXDateAndTime
				year: 2004
				month: 11
				day: 2
				hour: 14
				minute: 0
				second: 0
				offset: 0 hours.
	twoPmUTCInLocalTimeOfDetroit := twoPmInLondon utcOffset: -5 hours.
	nineAmInDetroit  := '2004-11-02T09:00:00-05:00' asLXDateAndTime.
	self assert:  twoPmUTCInLocalTimeOfDetroit = nineAmInDetroit.
	
! !

!LXDateAndTimeTest methodsFor: 'Tests' stamp: 'dtl 11/7/2004 13:01'!
testDateTimeDenotation2
  "LXDateAndTimeTest new testDateTimeDenotation2"
	
	 " Moscow is 3 hours ahead UTC, this offset to UTC is therefore positive. This example tests the correct interpretation of the DateAndTime denotation. "
	| lateEveningInLondon lateEveningInLocalTimeOfMoscow
	 localMoscowTimeFromDenotation |
	lateEveningInLondon := LXDateAndTime
				year: 2004
				month: 11
				day: 30
				hour: 23
				minute: 30
				second: 0
				offset: 0 hours.
	lateEveningInLocalTimeOfMoscow := lateEveningInLondon utcOffset: 3 hours.
	localMoscowTimeFromDenotation  := '2004-12-01T02:30:00+03:00' asLXDateAndTime.
	self assert:  lateEveningInLocalTimeOfMoscow = localMoscowTimeFromDenotation.
	
! !

!LXDateAndTimeTest methodsFor: 'Tests' stamp: 'fbs 11/5/2013 17:37'!
testErrorWhenDayIsAfterMonthEnd
	self
		should:
			[LXDateAndTime
				year: 2004
				month: 2
				day: 30]
		raise: Error.! !

!LXDateAndTimeTest methodsFor: 'Tests' stamp: 'fbs 11/5/2013 17:37'!
testErrorWhenDayIsBeforeMonthStart
	self
		should:
			[LXDateAndTime
				year: 2004
				month: 2
				day: -1]
		raise: Error.
	self
		should:
			[LXDateAndTime
				year: 2004
				month: 2
				day: 0]
		raise: Error.! !

!LXDateAndTimeTest methodsFor: 'Tests' stamp: 'dtl 3/23/2014 12:30'!
testFromString
	| fromString fromStringNoOffset fromStringUTC |
	fromString := LXDateAndTime fromString: '-1199-01-05T20:33:14.321-05:00'.
	self assert: (fromString printString = '-1199-01-05T20:33:14.321-05:00').
	
	"if no offset is provided, the local offset should be used"
	fromStringNoOffset := LXDateAndTime fromString: '-1199-01-05T20:33:14.321'.
	self assert: (fromStringNoOffset offset = LXDateAndTime localOffset).
	
	"if a string contains the UTC designator Z, the local offset should not be used"
	fromStringUTC := LXDateAndTime fromString: '2011-08-26T18:00:03Z'.
	self assert: (fromStringUTC printString = '2011-08-26T18:00:03+00:00').! !

!LXDateAndTimeTest methodsFor: 'Tests' stamp: 'nice 5/11/2012 22:56'!
testHash
	| date0 date1 date2 |
	date0 := LXDateAndTime unixEpoch.
	date1 := LXDateAndTime new ticks: (date0 + 1 hours) ticks offset: 0 hours.
	date2 := LXDateAndTime new ticks: (date0 - 2 hours) ticks offset: -3 hours.
	self assert: (date1 = date2) ==> [date1 hash = date2 hash]! !

!LXDateAndTimeTest methodsFor: 'Tests' stamp: 'brp 1/7/2004 15:37'!
testInstanceCreation
	| t |
	t := LXDateAndTime 
			year: 1 month: 1 day: 2 
			hour: 2 minute: 3 second: 4 nanoSecond: 5 
			offset: 6 hours.
	self 
		assert: (t julianDayNumber = 1721427);
		assert: (t offset = 6 hours);
		assert: (t hour = 2);
		assert: (t minute = 3);
		assert: (t second = 4);
		assert: (t nanoSecond = 5).
		
! !

!LXDateAndTimeTest methodsFor: 'Tests' stamp: 'nk 3/12/2004 11:06'!
testMonotonicity
	| t1 t2 t3 t4 |
	t1 := LXDateAndTime now.
	t2 := LXDateAndTime now.
	(Delay forMilliseconds: 1000) wait.
	t3 := LXDateAndTime now.
	t4 := LXDateAndTime now.
	self
		assert: (	t1 <= t2);
		assert: (	t2 < t3);
		assert: (	t3 <= t4).
! !

!LXDateAndTimeTest methodsFor: 'Tests' stamp: 'dtl 3/23/2014 12:30'!
testPrecision
	"Verify that the clock is returning a value with accuracy of better than 1 second.  For now it seems sufficient to get two values and verify they are not the same."
	self
		assert: (LXDateAndTime now ~= LXDateAndTime now)
! !

!LXDateAndTimeTest methodsFor: 'Tests' stamp: 'klc 9/9/2010 15:55'!
testPrintString
	"(self new setTestSelector: #testPrintString) debug"
	| dt dtNoOffset |
	dt :=LXDateAndTime
		year: 2004
		month: 11
		day: 2
		hour: 14
		minute: 3
		second: 5
		nanoSecond: 12345
		offset: (Duration seconds: (5 * 3600)).
	self assert: dt printString = '2004-11-02T14:03:05.000012345+05:00'.
	
	self assert: ('2002-05-16T17:20:45.1+01:01' asLXDateAndTime printString = '2002-05-16T17:20:45.1+01:01').
	self assert:	(' 2002-05-16T17:20:45.02+01:01' asLXDateAndTime printString = '2002-05-16T17:20:45.02+01:01').  
	self assert:	('2002-05-16T17:20:45.000000009+01:01' asLXDateAndTime printString =  '2002-05-16T17:20:45.000000009+01:01').
	self assert: ('2002-05-16T17:20:45+00:00' asLXDateAndTime printString = '2002-05-16T17:20:45+00:00' ).
	self assert: (' 2002-05-16T17:20:45+01:57' asLXDateAndTime printString = '2002-05-16T17:20:45+01:57').
	self assert: (' 2002-05-16T17:20:45-02:34' asLXDateAndTime printString = '2002-05-16T17:20:45-02:34').
	self assert: ('2002-05-16T17:20:45+00:00' asLXDateAndTime printString = '2002-05-16T17:20:45+00:00').
	self assert: ('1997-04-26T01:02:03+01:02:3' asLXDateAndTime printString = '1997-04-26T01:02:03+01:02:3').
	"When no offset is provided, the local one is used"
	dtNoOffset := '2002-05-16T17:20' asLXDateAndTime.
	self assert: (('2002-05-16T17:20:00*' match: dtNoOffset printString) and: [dtNoOffset offset = LXDateAndTime localOffset]).
! !

!LXDateAndTimeTest methodsFor: 'Tests' stamp: 'spd 8/5/2010 09:19'!
testReadFrom
self assert: ((LXDateAndTime readFrom: '-1199-01-05T20:33:14.321-05:00' readStream) printString = '-1199-01-05T20:33:14.321-05:00').
 ! !

!LXDateAndTimeTest methodsFor: 'Tests' stamp: 'brp 1/7/2004 15:43'!
testSmalltalk80Accessors
	| t |
	t := LXDateAndTime 
			year: 1 month: 1 day: 2 
			hour: 2 minute: 3 second: 4 nanoSecond: 5 
			offset: 6 hours.
	self 
		assert: (t hours = t hours);
		assert: (t minutes = t minute);
		assert: (t seconds = t second).
! !

!LXDateAndTimeTest methodsFor: 'Tests' stamp: 'BG 11/7/2004 12:18'!
testTimeZoneEquivalence
  "LXDateAndTimeTest new testTimeZoneEquivalence"
	"When the clock on the wall in Detroit says 9:00am, the clock on the wall
	in London says 2:00pm. The Duration difference between the corresponding
	DateAndTime values should be zero."
	
	 " Detroit is 5 hours behind UTC, this offset to UTC is therefore written with a minus sign. This example tests both the correct interpretation of the DateAndTime denotation and correct DateAndTime arithmetics. "
	| twoPmInLondon nineAmInDetroit durationDifference |
	twoPmInLondon := '2004-11-02T14:00:00+00:00' asLXDateAndTime.
	nineAmInDetroit  := '2004-11-02T09:00:00-05:00' asLXDateAndTime.
	durationDifference := twoPmInLondon - nineAmInDetroit.
	self assert: durationDifference asSeconds = 0.
	self assert: twoPmInLondon = nineAmInDetroit
! !

!LXDateAndTimeTest methodsFor: 'Tests' stamp: 'BG 11/7/2004 12:17'!
testTimeZoneEquivalence2
  "LXDateAndTimeTest new testTimeZoneEquivalence2"
	"This example demonstates the fact that
        2004-05-24T22:40:00  UTC  is
        2004-05-25T01:40:00  in Moscow
     (Moscow is 3 hours ahead of UTC)  "
	| thisMoment thisMomentInMoscow |
    thisMoment := LXDateAndTime year: 2004 month: 5 day: 24 hour: 22 minute: 40.
    thisMomentInMoscow := thisMoment utcOffset: 3 hours.
	self assert: (thisMoment - thisMomentInMoscow) asSeconds = 0.
	self assert: thisMoment = thisMomentInMoscow
! !


!LXTimeStamp methodsFor: 'squeak protocol' stamp: 'dtl 4/16/2014 19:25'!
asDateAndTime
	"Answer the receiver as an instance of DateAndTime."

	^ self asLXDateAndTime asDateAndTime! !

!LXTimeStamp methodsFor: 'squeak protocol' stamp: 'dtl 4/16/2014 19:22'!
asTimeStamp
	"Answer the receiver as an instance of TimeStamp."

	^ self asDateAndTime asTimeStamp! !

!LXTimeStamp methodsFor: 'squeak protocol' stamp: 'brp 7/27/2003 17:14'!
date
	"Answer the date of the receiver."

	^ self asDate! !

!LXTimeStamp methodsFor: 'squeak protocol' stamp: 'brp 7/27/2003 17:17'!
dateAndTime
	"Answer a two element Array containing the receiver's date and time."

	^ Array with: self date with: self time! !

!LXTimeStamp methodsFor: 'squeak protocol' stamp: 'brp 7/27/2003 17:19'!
minusDays: anInteger
	"Answer a TimeStamp which is anInteger days before the receiver."

	^ self - (anInteger days)! !

!LXTimeStamp methodsFor: 'squeak protocol' stamp: 'brp 7/27/2003 17:19'!
minusSeconds: anInteger
	"Answer a TimeStamp which is anInteger number of seconds before the receiver."

	^ self - (anInteger seconds)! !

!LXTimeStamp methodsFor: 'squeak protocol' stamp: 'brp 7/27/2003 17:18'!
plusDays: anInteger
	"Answer a TimeStamp which is anInteger days after the receiver."

	^ self + (anInteger days)! !

!LXTimeStamp methodsFor: 'squeak protocol' stamp: 'brp 7/27/2003 17:19'!
plusSeconds: anInteger
	"Answer a TimeStamp which is anInteger number of seconds after the receiver."

	^ self + (anInteger seconds)! !

!LXTimeStamp methodsFor: 'squeak protocol' stamp: 'brp 7/27/2003 17:17'!
printOn: aStream 
	"Print receiver's date and time on aStream."

	aStream 
		nextPutAll: self date printString;
		space;
		nextPutAll: self time printString.! !

!LXTimeStamp methodsFor: 'squeak protocol' stamp: 'brp 7/27/2003 17:17'!
storeOn: aStream 

	aStream 
		print: self printString;
		nextPutAll: ' asTimeStamp'! !

!LXTimeStamp methodsFor: 'squeak protocol' stamp: 'brp 7/27/2003 17:15'!
time
	"Answer the time of the receiver."

	^ self asTime! !

!LXTimeStamp methodsFor: 'transitional' stamp: 'dtl 4/16/2014 19:23'!
asLXDateAndTime
	"Answer the receiver as an instance of DateAndTime."

	^ LXDateAndTime utcMicroseconds: utcMicroseconds offset: self offsetSeconds
! !

!LXTimeStamp methodsFor: 'transitional' stamp: 'dtl 4/16/2014 19:22'!
asLXTimeStamp
	"Answer the receiver as an instance of TimeStamp."

	^ self! !


!LXTimeStamp class methodsFor: 'squeak protocol' stamp: 'dtl 7/13/2009 13:45'!
current

	| ts ticks |
	ts := super now asTimeStamp.
	ticks := ts ticks.
	ticks at: 3 put: 0.
	ts ticks: ticks offset: ts offset.
	^ ts
! !

!LXTimeStamp class methodsFor: 'ansi protocol' stamp: 'fbs 4/20/2004 14:22'!
now
	"Answer the current date and time as a TimeStamp."

	^self current! !


!TimeStamp methodsFor: 'LX-Kernel-Chronology' stamp: 'dtl 4/16/2014 19:34'!
asLXTimeStamp

	^self asLXDateAndTime asLXTimeStamp! !


!Timespan methodsFor: 'LX-Kernel-Chronology' stamp: 'dtl 3/23/2014 13:37'!
asLXDateAndTime

	^ start asLXDateAndTime! !

LXDateAndTime initialize!
"Postscript:
Ensure that the object history mark process uses DateAndTime>>floor rather than an earlier implementation that relied on a named instance variable."

ObjectHistory current restartMarkProcess.
!

