'From Squeak4.5 of 21 May 2014 [latest update: #13771] on 21 May 2014 at 9:41:53 pm'!
"Change Set:		LX-3
Date:			21 May 2014
Author:			David T. Lewis

DateAndTime and TimeStamp are now inactive, having been replaced by LXDateAndTime and LXTimeStamp. Load copies of the reimplemented LXDateAndTime and LXTimeStamp, renamed as DateAndTime and TimeStamp, to replace the original implementations of DateAndTime now.

At this point, DateAndTime is an inactive copy of LXDateAndTime. The next update will activate the new DateAndTime implementation."!

Magnitude subclass: #DateAndTime
	instanceVariableNames: 'utcMicroseconds localOffsetSeconds'
	classVariableNames: 'ClockProvider DaysSinceEpoch LastMilliSeconds LastTick LastTickSemaphore LocalTimeZone MilliSecondOffset OffsetsAreValid'
	poolDictionaries: 'ChronologyConstants'
	category: 'Kernel-Chronology'!

!DateAndTime commentStamp: 'brp 5/13/2003 08:07' prior: 0!
I represent a point in UTC time as defined by ISO 8601. I have zero duration.


My implementation uses three SmallIntegers
 and a Duration:
jdn		- julian day number.
seconds	- number of seconds since midnight.
nanos	- the number of nanoseconds since the second.

offset	- duration from UTC.

The nanosecond attribute is almost always zero but it defined for full ISO compliance and is suitable for timestamping.
!


!DateAndTime methodsFor: 'ansi protocol' stamp: 'dtl 3/31/2014 21:10'!
+ operand
	"operand conforms to protocol Duration"

	^ self class
		utcMicroseconds: operand asDuration asNanoSeconds / 1000 + utcMicroseconds
		offset: localOffsetSeconds
! !

!DateAndTime methodsFor: 'ansi protocol' stamp: 'dtl 3/23/2014 13:58'!
- operand
	"operand conforms to protocol DateAndTime or protocol Duration"

	^ (operand respondsTo: #asLXDateAndTime)
		ifTrue: 
			[ | lticks rticks |
			lticks := self asLocal ticks.
	
		rticks := operand asLXDateAndTime asLocal ticks.
			Duration
 				seconds: (SecondsInDay *(lticks first - rticks first)) + 
							(lticks second - rticks second)
 				nanoSeconds: (lticks third - rticks third) ]
	
	ifFalse:
		
 	[ self + (operand negated) ]
! !

!DateAndTime methodsFor: 'ansi protocol' stamp: 'dtl 5/13/2014 19:36'!
< comparand
	"comparand conforms to protocol DateAndTime,
	or can be converted into something that conforms."

	utcMicroseconds ifNil: [^true]. self flag: #FIXME. "dtl transitional hack"
	^utcMicroseconds < comparand asLXDateAndTime utcMicroseconds
! !

!DateAndTime methodsFor: 'ansi protocol' stamp: 'dtl 3/23/2014 22:51'!
= aDateAndTimeOrTimeStamp
	"Equal if the absolute time values match, regardless of local time transform"
	self == aDateAndTimeOrTimeStamp ifTrue: [ ^ true ].
	((aDateAndTimeOrTimeStamp isKindOf: self class)
		or: [aDateAndTimeOrTimeStamp isKindOf: LXDateAndTime orOf: TimeStamp])
			ifFalse: [ ^ false ].
	^utcMicroseconds = aDateAndTimeOrTimeStamp utcMicroseconds! !

!DateAndTime methodsFor: 'ansi protocol' stamp: 'dtl 3/23/2014 13:51'!
asUTC

	^ self offset isZero
		ifTrue: [self]
		ifFalse: [self utcOffset: 0]
! !

!DateAndTime methodsFor: 'ansi protocol' stamp: 'dtl 3/23/2014 17:32'!
dayOfWeek

	"Sunday=1, ... , Saturday=7"

	^ (self julianDayNumber + 1 rem: 7) + 1
! !

!DateAndTime methodsFor: 'ansi protocol' stamp: 'dtl 3/23/2014 22:39'!
hash
	| totalSeconds |
	totalSeconds := self getSeconds - self offsetSeconds.
	^ ((totalSeconds // 86400 + self julianDayNumber) hashMultiply bitXor: totalSeconds \\
86400) bitXor: self nanoSecond! !

!DateAndTime methodsFor: 'ansi protocol' stamp: 'dtl 3/23/2014 17:59'!
hour24


	^ (Duration seconds: self getSeconds) hours! !

!DateAndTime methodsFor: 'ansi protocol' stamp: 'dtl 3/23/2014 18:00'!
minute


	^ (Duration seconds: self getSeconds) minutes! !

!DateAndTime methodsFor: 'ansi protocol' stamp: 'dtl 3/23/2014 22:59'!
offset

	^ Duration seconds: localOffsetSeconds! !

!DateAndTime methodsFor: 'ansi protocol' stamp: 'dtl 3/23/2014 22:56'!
offset: anOffset

	"Answer a <DateAndTime> equivalent to the receiver but with its local time 
	being offset from UTC by offset."

	^ self class utcMicroseconds: utcMicroseconds offset: anOffset asDuration asSeconds
! !

!DateAndTime methodsFor: 'ansi protocol' stamp: 'dtl 3/23/2014 18:00'!
second


	^ (Duration seconds: self getSeconds) seconds! !

!DateAndTime methodsFor: 'smalltalk-80' stamp: 'dtl 4/10/2014 08:07'!
asSeconds
	"Return the number of seconds since the Squeak epoch"

	self flag: #FIXME. "is this whole seconds or partial? UTC or local?"

	^ utcMicroseconds / 1000000 + self class epochOffset
! !

!DateAndTime methodsFor: 'squeak protocol' stamp: 'dtl 3/23/2014 22:38'!
asDuration
	"Answer the duration since midnight."

	^ Duration seconds: self getSeconds nanoSeconds: self nanoSecond
! !

!DateAndTime methodsFor: 'squeak protocol' stamp: 'dtl 4/18/2014 11:01'!
asLXDateAndTime

	^ LXDateAndTime utcMicroseconds: utcMicroseconds offset: localOffsetSeconds! !

!DateAndTime methodsFor: 'squeak protocol' stamp: 'dtl 3/23/2014 22:38'!
asTime


	^ Time seconds: self getSeconds nanoSeconds: self nanoSecond
! !

!DateAndTime methodsFor: 'squeak protocol' stamp: 'dtl 3/23/2014 13:20'!
asTimeStamp

	^ self
		asDateAndTime "FIXME LX hack for test support"
		as: TimeStamp! !

!DateAndTime methodsFor: 'squeak protocol' stamp: 'dtl 3/23/2014 17:32'!
dayMonthYearDo: aBlock
	"Evaluation the block with three arguments: day month, year."

	| l n i j dd mm yyyy |
	l := self julianDayNumber + 68569.
	n := 4 * l // 146097.
	l := l - (146097 * n + 3 // 4).
	i := 4000 * (l + 1) // 1461001.
	l := l - (1461 * i // 4) + 31.
	j := 80 * l // 2447.
	dd := l - (2447 * j // 80).
	l := j // 11.
	mm := j + 2 - (12 * l).
	yyyy := 100 * (n - 49) + i + l.

	^ aBlock
		value: dd
		value: mm
		value: yyyy! !

!DateAndTime methodsFor: 'squeak protocol' stamp: 'dtl 5/21/2014 21:40'!
floor
	"Answer a copy with magnitude rounded down to the nearest whole second"
	^self class
		utcMicroseconds: utcMicroseconds - (utcMicroseconds \\ 1000000)
		offset: localOffsetSeconds! !

!DateAndTime methodsFor: 'squeak protocol' stamp: 'dtl 4/8/2014 20:10'!
julianDayNumber

	| posixDays posixSeconds localSeconds negativeDays |
	posixSeconds := utcMicroseconds / 1000000.
	localSeconds := posixSeconds + localOffsetSeconds.
	negativeDays := 0.
	localSeconds < 0 ifTrue: [ "normalize"
			negativeDays := localSeconds // SecondsInDay.
			localSeconds := negativeDays - 1 * SecondsInDay + localSeconds].
	posixDays := (localSeconds / SecondsInDay) asInteger.
	^posixDays + self posixEpochJulianDays - negativeDays.
! !

!DateAndTime methodsFor: 'squeak protocol' stamp: 'dtl 3/23/2014 22:59'!
makeUTC
	"Make the receiver's timezone UTC."
	localOffsetSeconds := 0! !

!DateAndTime methodsFor: 'squeak protocol' stamp: 'dtl 3/23/2014 17:33'!
midnight
	"Answer a DateAndTime starting at midnight of the same timezone offset as the receiver."
	^ self class basicNew
		setJdn: self julianDayNumber
		seconds: 0
		nano: 0
		offset: self offset! !

!DateAndTime methodsFor: 'squeak protocol' stamp: 'dtl 4/4/2014 20:19'!
nanoSecond

	^utcMicroseconds \\ 1000000 * 1000
! !

!DateAndTime methodsFor: 'squeak protocol' stamp: 'dtl 3/23/2014 14:42'!
printOn: aStream withLeadingSpace: printLeadingSpaceToo
	"Print as per ISO 8601 sections 5.3.3 and 5.4.1.
	If printLeadingSpaceToo is false, prints either:
		'YYYY-MM-DDThh:mm:ss.s+ZZ:zz:z' (for positive years) or '-YYYY-MM-DDThh:mm:ss.s+ZZ:zz:z' (for negative years)
	If printLeadingSpaceToo is true, prints either:
		' YYYY-MM-DDThh:mm:ss.s+ZZ:zz:z' (for positive years) or '-YYYY-MM-DDThh:mm:ss.s+ZZ:zz:z' (for negative years)
	"

	self printYMDOn: aStream withLeadingSpace: printLeadingSpaceToo.
	aStream nextPut: $T.
	self printHMSOn: aStream.
	self nanoSecond ~= 0 ifTrue:
		[ | z ps |
		ps := self nanoSecond printString padded: #left to: 9 with: $0.
		z := ps findLast: [ :c | c asciiValue > $0 asciiValue ].
		(z > 0) ifTrue: [aStream nextPut: $.].
		ps from: 1 to: z do: [ :c | aStream nextPut: c ] ].
	aStream
		nextPut: (self offset positive ifTrue: [$+] ifFalse: [$-]);
		nextPutAll: (self offset hours abs asString padded: #left to: 2 with: $0);
		nextPut: $:;
		nextPutAll: (self offset minutes abs asString padded: #left to: 2 with: $0).
	self offset seconds = 0 ifFalse:
		[ aStream
			nextPut: $:;
			nextPutAll: (self offset seconds abs truncated asString) ].
! !

!DateAndTime methodsFor: 'squeak protocol' stamp: 'dtl 3/23/2014 23:00'!
setJdn: jdn seconds: s nano: n offset: o

	localOffsetSeconds :=  o asSeconds.
	utcMicroseconds := self posixMicrosecondsJdn: jdn seconds: s nanos: n offset: localOffsetSeconds.

! !

!DateAndTime methodsFor: 'squeak protocol' stamp: 'dtl 3/23/2014 13:57'!
to: anEnd
	"Answer a Timespan. anEnd conforms to protocol DateAndTime or protocol Timespan"

	^ Timespan starting: self ending: (anEnd asLXDateAndTime)
! !

!DateAndTime methodsFor: 'squeak protocol' stamp: 'dtl 3/23/2014 13:57'!
to: anEnd by: aDuration
	"Answer a Timespan. anEnd conforms to protocol DateAndTime or protocol Timespan"

	^ (Schedule starting: self ending: (anEnd asLXDateAndTime))
		schedule: (Array with: aDuration asDuration);
		yourself
! !

!DateAndTime methodsFor: 'private' stamp: 'dtl 3/23/2014 22:38'!
hasEqualTicks: aDateAndTime
	
	^ (self julianDayNumber = aDateAndTime julianDayNumber)
		and: [ (self getSeconds = aDateAndTime secondsSinceMidnight)
			and: [ self nanoSecond = aDateAndTime nanoSecond ] ]

! !

!DateAndTime methodsFor: 'private' stamp: 'dtl 4/9/2014 22:24'!
normalize: i ticks: ticks base: base

	| tick div quo rem |
	tick := ticks at: i.
	div := tick asInteger digitDiv: base neg: tick negative.
	quo := (div at: 1) normalize.
	rem := (div at: 2) normalize.
	rem < 0 ifTrue: [ quo := quo - 1. rem := base + rem ].
	ticks at: (i-1) put: ((ticks at: i-1) + quo).
	ticks at: i put: rem
! !

!DateAndTime methodsFor: 'private' stamp: 'dtl 3/23/2014 18:00'!
secondsSinceMidnight

	^ self getSeconds! !

!DateAndTime methodsFor: 'private' stamp: 'dtl 3/23/2014 22:39'!
ticks
	"Private - answer an array with our instance variables. Assumed to be UTC "

	^ Array with: self julianDayNumber with: self getSeconds with: self nanoSecond
! !

!DateAndTime methodsFor: 'private' stamp: 'dtl 3/23/2014 23:00'!
ticks: ticks offset: utcOffset
	"ticks is {julianDayNumber. secondCount. nanoSeconds}"

	| jdn s nanos |
	self normalize: 3 ticks: ticks base: NanosInSecond.
	self normalize: 2 ticks: ticks base: SecondsInDay.

	jdn	:= ticks at: 1.
	s := ticks at: 2.
	nanos := ticks at: 3.
	localOffsetSeconds := utcOffset asSeconds.
	utcMicroseconds := self posixMicrosecondsJdn: jdn seconds: s nanos: nanos offset: localOffsetSeconds.
! !

!DateAndTime methodsFor: 'transitional - temporary' stamp: 'dtl 4/16/2014 19:20'!
asLXTimeStamp

	^ self as: LXTimeStamp! !

!DateAndTime methodsFor: 'transitional - temporary' stamp: 'dtl 3/23/2014 14:56'!
asPosixSeconds

	^ (self - (self class posixEpoch offset: self offset)) asSeconds! !

!DateAndTime methodsFor: 'transitional - temporary' stamp: 'dtl 4/9/2014 22:21'!
getSeconds

	| posixDays posixSeconds localSeconds |
	posixSeconds := utcMicroseconds / 1000000.
	localSeconds := posixSeconds + localOffsetSeconds.
	localSeconds < 0 ifTrue: [localSeconds := localSeconds \\ SecondsInDay]. "normalize"
	posixDays := (localSeconds / SecondsInDay) asInteger.
	^localSeconds - (posixDays * SecondsInDay).
! !

!DateAndTime methodsFor: 'transitional - temporary' stamp: 'dtl 3/23/2014 22:59'!
offsetSeconds

	^localOffsetSeconds ifNil: [localOffsetSeconds := self offset asSeconds]! !

!DateAndTime methodsFor: 'transitional - temporary' stamp: 'dtl 3/23/2014 22:44'!
posixMicrosecondsJdn: j seconds: s nanos: n offset: o

	| days totalSeconds micros |
	days := j - self posixEpochJulianDays.
	totalSeconds := days * 24 * 60 * 60 + s.
	micros := totalSeconds * 1000000.
	^micros + (n / 1000)
! !

!DateAndTime methodsFor: 'transitional - temporary' stamp: 'dtl 3/23/2014 22:51'!
utcMicroseconds
	^utcMicroseconds! !

!DateAndTime methodsFor: 'DTL-initializing' stamp: 'dtl 3/28/2014 07:53'!
posixEpochJulianDays

	^self class daysFromSmalltalkEpochToPosixEpoch + SqueakEpoch! !

!DateAndTime methodsFor: 'DTL-initializing' stamp: 'dtl 3/24/2014 16:35'!
utcMicroseconds: microsecondsSincePosixEpoch offset: tzOffset

	utcMicroseconds := microsecondsSincePosixEpoch.
	localOffsetSeconds := tzOffset.
! !


!DateAndTime class methodsFor: 'ansi protocol' stamp: 'dtl 4/16/2014 20:31'!
now 
	"^ self nowWithOffset: self localOffset"
	^LXDateAndTime now! !

!DateAndTime class methodsFor: 'squeak protocol' stamp: 'dtl 3/28/2014 18:44'!
epoch
	"Answer a DateAndTime representing the Squeak epoch: 1 January 1901"

	| uSec |
	self flag: #FIXME. "see comment in fromSeconds: "
	uSec := self epochOffsetMicros negated.
	^ self utcMicroseconds: uSec offset: self localOffsetSeconds


"
	^ self julianDayNumber: SqueakEpoch
"! !

!DateAndTime class methodsFor: 'squeak protocol' stamp: 'dtl 3/23/2014 13:42'!
tomorrow

	^ self today asDate next asLXDateAndTime
! !

!DateAndTime class methodsFor: 'squeak protocol' stamp: 'dtl 3/23/2014 13:57'!
yesterday

	^ self today asDate previous asLXDateAndTime! !

!DateAndTime class methodsFor: 'DTL' stamp: 'dtl 3/28/2014 07:53'!
daysFromSmalltalkEpochToPosixEpoch

	^52 * 365 + (17 * 366)! !

!DateAndTime class methodsFor: 'DTL' stamp: 'dtl 4/10/2014 08:02'!
epochOffset
	"Elaspsed seconds from the Smalltalk epoch to the Posix epoch"
	^self daysFromSmalltalkEpochToPosixEpoch * SecondsInDay! !

!DateAndTime class methodsFor: 'DTL' stamp: 'dtl 3/28/2014 18:44'!
epochOffsetMicros
	"Elaspsed microseconds from the Smalltalk epoch to the Posix epoch"
	^self epochOffset * 1000000! !

!DateAndTime class methodsFor: 'DTL' stamp: 'dtl 3/28/2014 08:05'!
localOffsetSeconds
	^self localOffset asSeconds! !

!DateAndTime class methodsFor: 'DTL' stamp: 'dtl 3/23/2014 14:56'!
posixEpoch
	"Answer a DateAndTime representing the Posix epoch"

	^ self julianDayNumber: self posixEpochJulianDays ! !

!DateAndTime class methodsFor: 'DTL' stamp: 'dtl 3/24/2014 16:30'!
primPosixMicrosecondClockWithOffset
	"Answer an array with UTC microseconds since the Posix epoch and the
	current seconds offset from GMT in the local time zone."

	<primitive: 'primitiveUtcWithOffset'>

	^self primitiveFailed! !

!DateAndTime class methodsFor: 'DTL' stamp: 'dtl 3/23/2014 22:55'!
utcMicroseconds: microsecondsSincePosixEpoch offset: offsetSeconds

	^super new
		utcMicroseconds: microsecondsSincePosixEpoch
		offset: offsetSeconds! !


!LXDateAndTime methodsFor: 'squeak protocol' stamp: 'dtl 4/18/2014 11:07'!
asDateAndTime

	^DateAndTime utcMicroseconds: utcMicroseconds offset: localOffsetSeconds
! !

!LXDateAndTime methodsFor: 'squeak protocol' stamp: 'dtl 5/20/2014 23:39'!
floor
	"Answer a copy with magnitude rounded down to the nearest whole second"
	^self class
		utcMicroseconds: utcMicroseconds - (utcMicroseconds \\ 1000000)
		offset: localOffsetSeconds! !


!String methodsFor: '*Kernel-Chronology' stamp: 'dtl 3/23/2014 12:38'!
asLXDateAndTime

	"Convert from UTC format" 	^ LXDateAndTime fromString: self! !


!Time class methodsFor: 'LX-Kernel-Chronology' stamp: 'dtl 3/22/2014 19:40'!
primPosixMicrosecondClockWithOffsetStoredIn: resultArray
	"Answer an array with UTC microseconds since the Posix epoch and the
	current seconds offset from GMT in the local time zone, supplying a two
	element result array that will be populated in the primitive."

	<primitive: 'primitiveUtcWithOffset'>
	^#(0 0)! !


!TimeStamp methodsFor: 'squeak protocol' stamp: 'dtl 4/18/2014 10:57'!
asDateAndTime
	"Answer the receiver as an instance of DateAndTime."

	^ DateAndTime utcMicroseconds: utcMicroseconds offset: localOffsetSeconds! !

!TimeStamp methodsFor: 'transitional' stamp: 'dtl 5/13/2014 20:16'!
asLXDateAndTime
	"Answer the receiver as an instance of DateAndTime."

	utcMicroseconds ifNil: [^LXDateAndTime new].
	^ LXDateAndTime utcMicroseconds: utcMicroseconds offset: self offsetSeconds
! !

!TimeStamp methodsFor: 'transitional' stamp: 'dtl 4/16/2014 19:22'!
asTimeStamp
	"Answer the receiver as an instance of TimeStamp."

	^ self! !


!TimeStamp class methodsFor: 'ansi protocol' stamp: 'fbs 4/20/2004 14:22'!
now
	"Answer the current date and time as a TimeStamp."

	^self current! !

!TimeStamp class methodsFor: 'squeak protocol' stamp: 'dtl 7/13/2009 13:45'!
current

	| ts ticks |
	ts := super now asTimeStamp.
	ticks := ts ticks.
	ticks at: 3 put: 0.
	ts ticks: ticks offset: ts offset.
	^ ts
! !

TimeStamp removeSelector: #asLXTimeStamp!
