'From Squeak4.5 of 21 May 2014 [latest update: #13771] on 21 May 2014 at 9:59:20 pm'!
"Change Set:		LX-4
Date:			21 May 2014
Author:			David T. Lewis

Resume using DateAndTime instead of LXDateAndTime.

Change instance creation in LXDateAndTime to create DateAndTime instances instead.
In the postScript, have LXDateAndTime instances become DateAndTime.

Be patient, the initialization takes some time.
"!


!DateAndTime methodsFor: 'ansi protocol' stamp: 'brp 1/9/2004 05:39'!
- operand
	"operand conforms to protocol DateAndTime or protocol Duration"

	^ (operand respondsTo: #asDateAndTime)
		ifTrue: 
			[ | lticks rticks |
			lticks := self asLocal ticks.
	
		rticks := operand asDateAndTime asLocal ticks.
			Duration
 				seconds: (SecondsInDay *(lticks first - rticks first)) + 
							(lticks second - rticks second)
 				nanoSeconds: (lticks third - rticks third) ]
	
	ifFalse:
		
 	[ self + (operand negated) ]
! !

!DateAndTime methodsFor: 'ansi protocol' stamp: 'dtl 4/18/2014 11:23'!
< comparand
	"comparand conforms to protocol DateAndTime,
	or can be converted into something that conforms."

	^utcMicroseconds < comparand asDateAndTime utcMicroseconds
! !

!DateAndTime methodsFor: 'ansi protocol' stamp: 'dtl 4/18/2014 11:18'!
= aDateAndTimeOrTimeStamp
	"Equal if the absolute time values match, regardless of local time transform"
	self == aDateAndTimeOrTimeStamp ifTrue: [ ^ true ].
	((aDateAndTimeOrTimeStamp isKindOf: self class)
		or: [aDateAndTimeOrTimeStamp isKindOf: DateAndTime orOf: TimeStamp])
			ifFalse: [ ^ false ].
	^utcMicroseconds = aDateAndTimeOrTimeStamp utcMicroseconds! !

!DateAndTime methodsFor: 'squeak protocol' stamp: 'dtl 5/21/2014 21:40'!
floor
	"Answer a copy with magnitude rounded down to the nearest whole second"
	^self class
		utcMicroseconds: utcMicroseconds - (utcMicroseconds \\ 1000000)
		offset: localOffsetSeconds! !

!DateAndTime methodsFor: 'squeak protocol' stamp: 'brp 7/27/2003 15:50'!
to: anEnd
	"Answer a Timespan. anEnd conforms to protocol DateAndTime or protocol Timespan"

	^ Timespan starting: self ending: (anEnd asDateAndTime)
! !

!DateAndTime methodsFor: 'squeak protocol' stamp: 'brp 9/25/2003 15:57'!
to: anEnd by: aDuration
	"Answer a Timespan. anEnd conforms to protocol DateAndTime or protocol Timespan"

	^ (Schedule starting: self ending: (anEnd asDateAndTime))
		schedule: (Array with: aDuration asDuration);
		yourself
! !


!DateAndTime class methodsFor: 'ansi protocol' stamp: 'dtl 4/18/2014 10:44'!
now

	[ | timeArray |
	timeArray := self primPosixMicrosecondClockWithOffset.
	^ self utcMicroseconds: timeArray first offset: timeArray second]
		on: Error
		do: [ "Use old style primitive support"
			^self nowWithOffset: self localOffset]! !

!DateAndTime class methodsFor: 'smalltalk-80' stamp: 'dtl 4/18/2014 10:42'!
fromSeconds: seconds 
	"Answer a DateAndTime since the Squeak epoch: 1 January 1901"

	"dtl - Implemented for compatibiity with Squeak, but this is probably wrong"

	| s uSec offset |
	self flag: #FIXME. "local versus UTC? what should we set the offset to?"
	offset := self localOffsetSeconds.
	s := seconds - self epochOffset - offset..
	uSec := s * 1000000.
	^ self utcMicroseconds: uSec offset: offset
! !

!DateAndTime class methodsFor: 'squeak protocol' stamp: 'cmm 2/2/2012 19:13'!
julianDayNumber: anInteger offset: aDuration 
	^ self basicNew
		ticks: anInteger days ticks
		offset: aDuration ;
		 yourself! !

!DateAndTime class methodsFor: 'squeak protocol' stamp: 'brp 8/24/2003 12:19'!
tomorrow

	^ self today asDate next asDateAndTime
! !

!DateAndTime class methodsFor: 'squeak protocol' stamp: 'dtl 4/18/2014 10:48'!
year: year month: month day: day hour: hour minute: minute second: second nanoSecond: nanoCount offset: offset
	"Return a DateAndTime"

	| monthIndex daysInMonth p q r s julianDayNumber |

	monthIndex := month isInteger ifTrue: [month] ifFalse: [Month indexOfMonth: month].
	daysInMonth := Month
		daysInMonth: monthIndex
		forYear: year.
	day < 1 ifTrue: [self error: 'day may not be zero or negative'].
	day > daysInMonth ifTrue: [self error: 'day is after month ends']. 	
	
	p := (monthIndex - 14) quo: 12.
	q := year + 4800 + p.
	r := monthIndex - 2 - (12 * p).
	s := (year + 4900 + p) quo: 100.

	julianDayNumber :=
 		( (1461 * q) quo: 4 ) +
			( (367 * r) quo: 12 ) -
 				( (3 * s) quo: 4 ) +
 					( day - 32075 ).

	^self basicNew
		setJdn: julianDayNumber 
		seconds: hour * 60 + minute * 60 + second - offset asSeconds
		nano: nanoCount
		offset: offset;
		yourself! !

!DateAndTime class methodsFor: 'squeak protocol' stamp: 'brp 8/24/2003 12:19'!
yesterday

	^ self today asDate previous asDateAndTime! !


!LXDateAndTime methodsFor: 'squeak protocol' stamp: 'dtl 5/20/2014 23:39'!
floor
	"Answer a copy with magnitude rounded down to the nearest whole second"
	^self class
		utcMicroseconds: utcMicroseconds - (utcMicroseconds \\ 1000000)
		offset: localOffsetSeconds! !


!LXDateAndTime class methodsFor: 'initialize-release' stamp: 'dtl 5/14/2014 21:13'!
initialize

	super initialize.

	ClockProvider := Time.
	LastTickSemaphore := Semaphore forMutualExclusion.
	LastMilliSeconds := 0.
	LastTick := 0.
	Smalltalk addToStartUpList: self.
	self startUp: true.

	"Postscript:
	Convert all instances of LXDateAndTime and LXTimeStamp to the equivalent DateAndTime and TimeStamp."

	Smalltalk garbageCollect.
	LXDateAndTime allInstancesDo: [: e | e become: e asDateAndTime].
	LXTimeStamp allInstancesDo: [: e | e become: e asTimeStamp].
	Smalltalk garbageCollect.


! !


!TimeStamp methodsFor: 'squeak protocol' stamp: 'dtl 4/16/2014 19:22'!
asTimeStamp
	"Answer the receiver as an instance of TimeStamp."

	^ self! !

Time class removeSelector: #primPosixMicrosecondClockWithOffsetStoredIn:!
LXDateAndTime initialize!
"Postscript:
Convert all instances of LXDateAndTime and LXTimeStamp to the equivalent DateAndTime and TimeStamp."

Smalltalk garbageCollect.
LXDateAndTime allInstancesDo: [: e | e become: e asDateAndTime].
LXTimeStamp allInstancesDo: [: e | e become: e asTimeStamp].
Smalltalk garbageCollect.

ObjectHistory current restartMarkProcess.
!

