'From Squeak4.5 of 21 May 2014 [latest update: #13771] on 21 May 2014 at 10:23:53 pm'!
"Change Set:		LX-5
Date:			21 May 2014
Author:			David T. Lewis

UTC based DateAndTime - various functional and performance updates. Remove left over LXDateAndTime and related classes and tests.

The package is now suitable as a replacement for the original jdn/seconds/nanos/offset implementation of DateAndTime.

Test failures remain in the Chronology test suite. These are tests that should be reviewed, as some may be documenting undesirable prior behavior.
"!

Magnitude subclass: #DateAndTime
	instanceVariableNames: 'utcMicroseconds localOffsetSeconds'
	classVariableNames: 'ClockProvider DaysSinceEpoch LastMilliSeconds LastTick LastTickSemaphore LocalTimeZone MilliSecondOffset OffsetsAreValid PosixEpochJulianDays'
	poolDictionaries: 'ChronologyConstants'
	category: 'Kernel-Chronology'!

!DateAndTime commentStamp: 'brp 5/13/2003 08:07' prior: 0!
I represent a point in UTC time as defined by ISO 8601. I have zero duration.


My implementation uses three SmallIntegers
 and a Duration:
jdn		- julian day number.
seconds	- number of seconds since midnight.
nanos	- the number of nanoseconds since the second.

offset	- duration from UTC.

The nanosecond attribute is almost always zero but it defined for full ISO compliance and is suitable for timestamping.
!


!DateAndTime methodsFor: 'ansi protocol' stamp: 'dtl 4/18/2014 12:19'!
- operand
	"operand conforms to protocol DateAndTime or protocol Duration"

	^ (operand respondsTo: #asDateAndTime)
		ifTrue: 
			[ | micros |
			micros := utcMicroseconds - operand asDateAndTime utcMicroseconds.
			Duration seconds: micros // 1000000 nanoSeconds: micros \\ 1000000 * 1000]
		ifFalse:
			[ self + (operand negated) ]
! !

!DateAndTime methodsFor: 'ansi protocol' stamp: 'dtl 5/20/2014 22:10'!
hash
	^utcMicroseconds hash! !

!DateAndTime methodsFor: 'ansi protocol' stamp: 'dtl 4/18/2014 13:35'!
hour24


	^ (Duration seconds: self getWholeSeconds) hours! !

!DateAndTime methodsFor: 'ansi protocol' stamp: 'dtl 4/18/2014 13:35'!
minute


	^ (Duration seconds: self getWholeSeconds) minutes! !

!DateAndTime methodsFor: 'ansi protocol' stamp: 'dtl 4/23/2014 08:19'!
offset: anOffset

	"Answer a <DateAndTime> equivalent to the receiver but with its local time 
	being offset from UTC by offset."

	| newOffset newMicros |
	self flag: #FIXME. "check the definition of this and of #utcOffset:"
	newOffset := anOffset asDuration asSeconds.
	newMicros := localOffsetSeconds - newOffset * 1000000 + utcMicroseconds.
	^ self class utcMicroseconds: newMicros offset: newOffset
! !

!DateAndTime methodsFor: 'ansi protocol' stamp: 'dtl 4/18/2014 13:35'!
second


	^ (Duration seconds: self getWholeSeconds) seconds! !

!DateAndTime methodsFor: 'squeak protocol' stamp: 'dtl 5/21/2014 21:40'!
floor
	"Answer a copy with magnitude rounded down to the nearest whole second"
	^self class
		utcMicroseconds: utcMicroseconds - (utcMicroseconds \\ 1000000)
		offset: localOffsetSeconds! !

!DateAndTime methodsFor: 'squeak protocol' stamp: 'dtl 4/18/2014 13:50'!
julianDayNumber

	| posixDays posixSeconds localSeconds negativeDays |
	posixSeconds := utcMicroseconds // 1000000.
	localSeconds := posixSeconds + localOffsetSeconds.
	negativeDays := 0.
	localSeconds < 0 ifTrue: [ "normalize"
			negativeDays := localSeconds // SecondsInDay.
			localSeconds := negativeDays * SecondsInDay + localSeconds].
	posixDays := localSeconds // SecondsInDay.
	^posixDays + PosixEpochJulianDays - negativeDays.
! !

!DateAndTime methodsFor: 'squeak protocol' stamp: 'dtl 4/18/2014 14:18'!
printHMSOn: aStream
	"Print just hh:mm:ss"

	| d |
	d := Duration seconds: self getWholeSeconds.
	aStream
		nextPutAll: (d hours asString padded: #left to: 2 with: $0);
		nextPut: $:;
		nextPutAll: (d minutes asString padded: #left to: 2 with: $0);
		nextPut: $:;
		nextPutAll: (d seconds asString padded: #left to: 2 with: $0).
! !

!DateAndTime methodsFor: 'squeak protocol' stamp: 'dtl 5/8/2014 19:15'!
printOn: aStream withLeadingSpace: printLeadingSpaceToo
	"Print as per ISO 8601 sections 5.3.3 and 5.4.1.
	If printLeadingSpaceToo is false, prints either:
		'YYYY-MM-DDThh:mm:ss.s+ZZ:zz:z' (for positive years) or '-YYYY-MM-DDThh:mm:ss.s+ZZ:zz:z' (for negative years)
	If printLeadingSpaceToo is true, prints either:
		' YYYY-MM-DDThh:mm:ss.s+ZZ:zz:z' (for positive years) or '-YYYY-MM-DDThh:mm:ss.s+ZZ:zz:z' (for negative years)
	"

	| nano |
	self printYMDOn: aStream withLeadingSpace: printLeadingSpaceToo.
	aStream nextPut: $T.
	self printHMSOn: aStream.
	(nano := self nanoSecond) ~= 0 ifTrue:
		[ | z ps |
		ps := nano asInteger printString padded: #left to: 9 with: $0.
		z := ps findLast: [ :c | c asciiValue > $0 asciiValue ].
		(z > 0) ifTrue: [aStream nextPut: $.].
		ps from: 1 to: z do: [ :c | aStream nextPut: c ] ].
	aStream
		nextPut: (self offset positive ifTrue: [$+] ifFalse: [$-]);
		nextPutAll: (self offset hours abs asString padded: #left to: 2 with: $0);
		nextPut: $:;
		nextPutAll: (self offset minutes abs asString padded: #left to: 2 with: $0).
	self offset seconds = 0 ifFalse:
		[ aStream
			nextPut: $:;
			nextPutAll: (self offset seconds abs truncated asString) ].
! !

!DateAndTime methodsFor: 'squeak protocol' stamp: 'dtl 4/20/2014 22:28'!
setJdn: jdn seconds: s nano: n offset: o

	localOffsetSeconds :=  o asSeconds.
	utcMicroseconds := self
				posixMicrosecondsJdn: jdn
				seconds: s - localOffsetSeconds
				nanos: n
				offset: localOffsetSeconds! !

!DateAndTime methodsFor: 'squeak protocol' stamp: 'dtl 4/23/2014 08:13'!
utcOffset: anOffset

	"Answer a <DateAndTime> equivalent to the receiver but offset from UTC by anOffset"

	self flag: #FIXME. "check the definition of this and of #offset:"
	^self utcMicroseconds: utcMicroseconds offset: anOffset asDuration asSeconds
! !

!DateAndTime methodsFor: 'converting' stamp: 'dtl 3/23/2014 14:56'!
asPosixSeconds

	^ (self - (self class posixEpoch offset: self offset)) asSeconds! !

!DateAndTime methodsFor: 'accessing' stamp: 'dtl 4/9/2014 22:21'!
getSeconds

	| posixDays posixSeconds localSeconds |
	posixSeconds := utcMicroseconds / 1000000.
	localSeconds := posixSeconds + localOffsetSeconds.
	localSeconds < 0 ifTrue: [localSeconds := localSeconds \\ SecondsInDay]. "normalize"
	posixDays := (localSeconds / SecondsInDay) asInteger.
	^localSeconds - (posixDays * SecondsInDay).
! !

!DateAndTime methodsFor: 'accessing' stamp: 'dtl 4/18/2014 13:33'!
getWholeSeconds

	| posixDays posixSeconds localSeconds |
	posixSeconds := utcMicroseconds // 1000000.
	localSeconds := posixSeconds + localOffsetSeconds.
	localSeconds < 0 ifTrue: [localSeconds := localSeconds \\ SecondsInDay]. "normalize"
	posixDays := localSeconds // SecondsInDay.
	^localSeconds - (posixDays * SecondsInDay).
! !

!DateAndTime methodsFor: 'accessing' stamp: 'dtl 3/23/2014 22:59'!
offsetSeconds

	^localOffsetSeconds ifNil: [localOffsetSeconds := self offset asSeconds]! !

!DateAndTime methodsFor: 'accessing' stamp: 'dtl 3/23/2014 22:51'!
utcMicroseconds
	^utcMicroseconds! !

!DateAndTime methodsFor: 'initialize-release' stamp: 'dtl 3/28/2014 07:53'!
posixEpochJulianDays

	^self class daysFromSmalltalkEpochToPosixEpoch + SqueakEpoch! !

!DateAndTime methodsFor: 'initialize-release' stamp: 'dtl 4/18/2014 13:22'!
posixMicrosecondsJdn: j seconds: s nanos: n offset: o

	| days totalSeconds micros |
	days := j - PosixEpochJulianDays.
	totalSeconds := days * 24 * 60 * 60 + s.
	micros := totalSeconds * 1000000.
	^micros + (n / 1000)
! !

!DateAndTime methodsFor: 'initialize-release' stamp: 'dtl 3/24/2014 16:35'!
utcMicroseconds: microsecondsSincePosixEpoch offset: tzOffset

	utcMicroseconds := microsecondsSincePosixEpoch.
	localOffsetSeconds := tzOffset.
! !


!DateAndTime class methodsFor: 'squeak protocol' stamp: 'dtl 4/23/2014 18:53'!
new
	"Answer a DateAndTime representing the Squeak epoch: 1 January 1901"

	^ self utcMicroseconds: self epochOffsetMicros negated offset: 0

! !

!DateAndTime class methodsFor: 'squeak protocol' stamp: 'dtl 4/30/2014 23:06'!
readFrom: aStream

	| offsetSeconds ch yearMonthDay hourMinuteSecondNano offset |

	yearMonthDay := Date readYearMonthDayFrom: aStream.
	[aStream peek isDigit]
		whileFalse: [aStream next].
	hourMinuteSecondNano := Time readHourMinuteSecondNanoFrom: aStream.
	(aStream atEnd or: [('+-Z' includes: aStream peek) not])
		ifTrue: [ self flag: #FIXME.
				"Different unit tests have conflicting opinions as to whether the
				current local offset should be used as a default. However, the current
				local offset cannot be correct due to DST (offset is itself a function
				of the point in time). Therefore, defaulting to offset 0 is the only
				approach that can be done. Unit tests should be changed accordingly."
				offsetSeconds := 0.
				"offsetSeconds := self localOffset asSeconds"]
		ifFalse: [(aStream peekFor: $Z)
			ifTrue: [offsetSeconds := 0]
			ifFalse: [
				ch := aStream next.
				ch = $+ ifTrue: [ch := Character space].
				offset := Duration fromString: ch asString, '0:', aStream upToEnd, ':0'.
				offsetSeconds := offset asSeconds]].
	^ self
		year: yearMonthDay first
		month: yearMonthDay second
		day: yearMonthDay third
		hour: hourMinuteSecondNano first
		minute: hourMinuteSecondNano second
		second: hourMinuteSecondNano third
		nanoSecond: hourMinuteSecondNano fourth
		offsetSeconds: offsetSeconds


	"	'-1199-01-05T20:33:14.321-05:00' asDateAndTime
		' 2002-05-16T17:20:45.1+01:01' asDateAndTime

		' 2002-05-16T17:20:45.02+01:01' asDateAndTime

		' 2002-05-16T17:20:45.003+01:01' asDateAndTime

		' 2002-05-16T17:20:45.0004+01:01' asDateAndTime
  		' 2002-05-16T17:20:45.00005' asDateAndTime
		' 2002-05-16T17:20:45.000006+01:01' asDateAndTime

		' 2002-05-16T17:20:45.0000007+01:01' asDateAndTime
		' 2002-05-16T17:20:45.00000008-01:01' asDateAndTime   
		' 2002-05-16T17:20:45.000000009+01:01' asDateAndTime  
		' 2002-05-16T17:20:45.0000000001+01:01' asDateAndTime  

 		' 2002-05-16T17:20' asDateAndTime
		' 2002-05-16T17:20:45' asDateAndTime
		' 2002-05-16T17:20:45+01:57' asDateAndTime
 		' 2002-05-16T17:20:45-02:34' asDateAndTime
 		' 2002-05-16T17:20:45+00:00' asDateAndTime
		' 1997-04-26T01:02:03+01:02:3' asDateAndTime 
 	"! !

!DateAndTime class methodsFor: 'squeak protocol' stamp: 'dtl 4/18/2014 15:04'!
year: year month: month day: day hour: hour minute: minute second: second nanoSecond: nanoCount offset: offset
	"Return a DateAndTime"

	| offsetSeconds utcMicros |
	offsetSeconds := offset asSeconds.
	utcMicros := self
				utcMicrosecondsForYear: year
				month: month
				day: day
				hour: hour
				minute: minute
				second: second
				nanoSecond: nanoCount
				offsetSeconds: offsetSeconds.
	^ self utcMicroseconds: utcMicros offset: offsetSeconds! !

!DateAndTime class methodsFor: 'squeak protocol' stamp: 'dtl 4/18/2014 17:12'!
year: year month: month day: day hour: hour minute: minute second: second nanoSecond: nanoCount offsetSeconds: offsetSeconds
	"Return a DateAndTime"

	| utcMicros |
	utcMicros := self
				utcMicrosecondsForYear: year
				month: month
				day: day
				hour: hour
				minute: minute
				second: second
				nanoSecond: nanoCount
				offsetSeconds: offsetSeconds.
	^ self utcMicroseconds: utcMicros offset: offsetSeconds! !

!DateAndTime class methodsFor: 'initialize-release' stamp: 'dtl 4/18/2014 13:20'!
initialize

	super initialize.

	ClockProvider := Time.
	LastTickSemaphore := Semaphore forMutualExclusion.
	LastMilliSeconds := 0.
	LastTick := 0.
	PosixEpochJulianDays := 2440588.
	Smalltalk addToStartUpList: self.
	self startUp: true
! !

!DateAndTime class methodsFor: 'DTL' stamp: 'dtl 4/18/2014 13:22'!
posixEpoch
	"Answer a DateAndTime representing the Posix epoch"

	^ self julianDayNumber: PosixEpochJulianDays ! !

!DateAndTime class methodsFor: 'DTL' stamp: 'dtl 4/18/2014 15:03'!
utcMicrosecondsForYear: year month: month day: day hour: hour minute: minute second: second nanoSecond: nanoCount  offsetSeconds: offsetSeconds

	| monthIndex daysInMonth p q r s julianDayNumber posixDays seconds utcSeconds |

	monthIndex := month isInteger ifTrue: [month] ifFalse: [Month indexOfMonth: month].
	daysInMonth := Month
		daysInMonth: monthIndex
		forYear: year.
	day < 1 ifTrue: [self error: 'day may not be zero or negative'].
	day > daysInMonth ifTrue: [self error: 'day is after month ends']. 	
	
	p := (monthIndex - 14) quo: 12.
	q := year + 4800 + p.
	r := monthIndex - 2 - (12 * p).
	s := (year + 4900 + p) quo: 100.

	julianDayNumber :=
 		( (1461 * q) quo: 4 ) +
			( (367 * r) quo: 12 ) -
 				( (3 * s) quo: 4 ) +
 					( day - 32075 ).

	posixDays := julianDayNumber - PosixEpochJulianDays.
	seconds := hour * 60 + minute * 60 + second - offsetSeconds.
	utcSeconds := seconds + (posixDays * 24 * 3600).
	^ utcSeconds * 1000000 + (nanoCount / 1000)
! !


!Duration class methodsFor: 'squeak protocol' stamp: 'dtl 4/18/2014 17:23'!
readDayHourMinuteSecondNanaFrom: aStream
	"Formatted as per ANSI 5.8.2.16: [-]D:HH:MM:SS[.S]"

	| sign days hours minutes seconds nanos nanosBuffer |
	sign := (aStream peekFor: $-) ifTrue: [-1] ifFalse: [1].
	days := (aStream upTo: $:) asInteger sign: sign.
	hours := (aStream upTo: $:) asInteger sign: sign.
	minutes := (aStream upTo: $:) asInteger sign: sign.
	seconds := (aStream upTo: $.) asInteger sign: sign.
	nanosBuffer := '000000000' copy.
	nanos := WriteStream on: nanosBuffer.
	[aStream atEnd not and: [aStream peek isDigit]]
		whileTrue: [nanos nextPut: aStream next].
		
	^ { days . hours . minutes . seconds .nanosBuffer asInteger sign: sign }

	"	'0:00:00:00' asDuration
		'0:00:00:00.000000001' asDuration
		'0:00:00:00.999999999' asDuration
		'0:00:00:00.100000000' asDuration
		'0:00:00:00.001 ' asDuration
		'0:00:00:00.1' asDuration
		'0:00:00:01 ' asDuration
		'0:12:45:45' asDuration
		'1:00:00:00' asDuration
		'365:00:00:00' asDuration
		'-7:09:12:06.10' asDuration
		'+0:01:02:3' asDuration
 	"! !

!Duration class methodsFor: 'squeak protocol' stamp: 'dtl 4/18/2014 17:25'!
readFrom: aStream
	"Formatted as per ANSI 5.8.2.16: [-]D:HH:MM:SS[.S]"

	| dayHourMinuteSecondNano |
	dayHourMinuteSecondNano := self readDayHourMinuteSecondNanaFrom: aStream.
		
	^ self 
		days: dayHourMinuteSecondNano first 
		hours: dayHourMinuteSecondNano second 
		minutes: dayHourMinuteSecondNano third 
		seconds: dayHourMinuteSecondNano fourth 
		nanoSeconds: dayHourMinuteSecondNano fifth

	"	'0:00:00:00' asDuration
		'0:00:00:00.000000001' asDuration
		'0:00:00:00.999999999' asDuration
		'0:00:00:00.100000000' asDuration
		'0:00:00:00.001 ' asDuration
		'0:00:00:00.1' asDuration
		'0:00:00:01 ' asDuration
		'0:12:45:45' asDuration
		'1:00:00:00' asDuration
		'365:00:00:00' asDuration
		'-7:09:12:06.10' asDuration
		'+0:01:02:3' asDuration
 	"! !


!Time class methodsFor: 'smalltalk-80' stamp: 'dtl 4/18/2014 17:34'!
readFrom: aStream
	"Read a Time from the stream in the form:
		<hour>:<minute>:<second> <am/pm>

	<minute>, <second> or <am/pm> may be omitted.  e.g. 1:59:30 pm; 8AM; 15:30"

	| hourMinuteSecondNano |
	hourMinuteSecondNano := self readHourMinuteSecondNanoFrom: aStream.
	^ self 
		hour: hourMinuteSecondNano first 
		minute: hourMinuteSecondNano second 
		second: hourMinuteSecondNano third 
		nanoSecond: hourMinuteSecondNano fourth

	"Time readFrom: (ReadStream on: '2:23:09 pm')"! !

!Time class methodsFor: 'smalltalk-80' stamp: 'dtl 4/18/2014 17:32'!
readHourMinuteSecondNanoFrom: aStream
	"Read a Time from the stream in the form:
		<hour>:<minute>:<second> <am/pm>

	<minute>, <second> or <am/pm> may be omitted.  e.g. 1:59:30 pm; 8AM; 15:30"

	| hour minute second ampm nanos nanosBuffer |
	hour := Integer readFrom: aStream.
	minute := 0.
	second := 0.
	nanosBuffer := '000000000' copy.
	nanos := WriteStream on: nanosBuffer.
	(aStream peekFor: $:) 
		ifTrue: [
			minute := Integer readFrom: aStream.
			(aStream peekFor: $:) 
				ifTrue: [
					second := Integer readFrom: aStream.
					(aStream peekFor: $.)
						ifTrue: [
							[aStream atEnd not and: [aStream peek isDigit]]
								whileTrue: [nanos nextPut: aStream next]]]].
	aStream skipSeparators.
	(aStream atEnd not and: ['PApa' includes: aStream peek]) ifTrue: 
		[ampm := aStream next asLowercase.
		(ampm = $p and: [hour < 12]) ifTrue: [hour := hour + 12].
		(ampm = $a and: [hour = 12]) ifTrue: [hour := 0].
		(aStream peekFor: $m) ifFalse: [aStream peekFor: $M ]].
	^ { hour . minute . second . nanosBuffer asInteger }

	"Time readFrom: (ReadStream on: '2:23:09 pm')"! !


!TimeStamp class methodsFor: 'squeak protocol' stamp: 'dtl 4/18/2014 11:39'!
current

	^self now! !


!Date class methodsFor: 'squeak protocol' stamp: 'dtl 4/18/2014 17:01'!
readFrom: aStream 
	"Read a Date from the stream in any of the forms:  
		<day> <month> <year>		(15 April 1982; 15-APR-82; 15.4.82; 15APR82)  
		<month> <day> <year>		(April 15, 1982; 4/15/82)
		<year>-<month>-<day>			(1982-04-15) (ISO8601)"
	| ymd |
	ymd := self readYearMonthDayFrom: aStream.
	^ self
		year: ymd first
		month: ymd second
		day: ymd third
! !

!Date class methodsFor: 'squeak protocol' stamp: 'dtl 4/18/2014 16:57'!
readYearMonthDayFrom: aStream 
	"Read Date information from the stream in any of the forms:  
		<day> <month> <year>		(15 April 1982; 15-APR-82; 15.4.82; 15APR82)  
		<month> <day> <year>		(April 15, 1982; 4/15/82)
		<year>-<month>-<day>			(1982-04-15) (ISO8601)"
	| day month year parsedNumber prefix monthIndex |
	aStream peek = $-
		ifTrue: [prefix := -1]
		ifFalse: [prefix := 1].
	[aStream peek isAlphaNumeric]
		whileFalse: [aStream skip: 1].
	aStream peek isDigit
		ifTrue: [
			parsedNumber := (Integer readFrom: aStream) * prefix.
			(parsedNumber < 0 or: [parsedNumber > 31])
				ifTrue: [year := parsedNumber]].
	[aStream peek isAlphaNumeric]
		whileFalse: [aStream skip: 1].
	aStream peek isLetter
		ifTrue: ["MM-DD-YY or DD-MM-YY or YY-MM-DD"
			month := WriteStream on: (String new: 10).
			[aStream peek isLetter]
				whileTrue: [month nextPut: aStream next].
			month := month contents.
			[aStream peek isAlphaNumeric]
				whileFalse: [aStream skip: 1].
			parsedNumber isNil
				ifTrue: ["MM DD YY"
					day := Integer readFrom: aStream]
				ifFalse: [
					year isNil
						ifTrue: ["DD MM YY"
							day := parsedNumber]]]
		ifFalse: ["MM-DD-YY or DD-MM-YY or YY-MM-DD"
			year isNil 
				ifTrue: ["MM-DD-YY or DD-MM-YY"
					parsedNumber > 12
						ifTrue: ["DD-MM-YY"
							day := parsedNumber.
							monthIndex := Integer readFrom: aStream.
							"month := Month nameOfMonth: (Integer readFrom: aStream)"]
						ifFalse: ["MM-DD-YY"
							monthIndex := parsedNumber.
							"month := Month nameOfMonth: parsedNumber."
							day := Integer readFrom: aStream]]
				ifFalse: ["YY-MM-DD"
					monthIndex := Integer readFrom: aStream.
					"month := Month nameOfMonth: (Integer readFrom: aStream)"]].
	[aStream peek isAlphaNumeric]
		whileFalse: [aStream skip: 1].
	year isNil
		ifTrue: [year := Integer readFrom: aStream]
		ifFalse: [day := Integer readFrom: aStream].
	(year < 100 and: [year >= 0]) 
		ifTrue: [
			year < 69 
				ifTrue: [	year := 2000 + year]
				ifFalse: [year := 1900 + year]].

	monthIndex ifNil: [monthIndex := Month indexOfMonth: month].
	^ { year . monthIndex . day }
! !

Timespan removeSelector: #asLXDateAndTime!
TimeStamp class removeSelector: #now!
TimeStamp removeSelector: #asLXDateAndTime!
String removeSelector: #asLXDateAndTime!
DateAndTime initialize!
DateAndTime removeSelector: #asLXDateAndTime!
DateAndTime removeSelector: #asLXTimeStamp!
Smalltalk removeClassNamed: #LXDateAndTime!
Smalltalk removeClassNamed: #LXDateAndTimeConversionTest!
Smalltalk removeClassNamed: #LXDateAndTimeEpochTest!
Smalltalk removeClassNamed: #LXDateAndTimeLeapTest!
Smalltalk removeClassNamed: #LXDateAndTimeTest!
Smalltalk removeClassNamed: #LXTimeStamp!
